/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <glib/gstdio.h>
#include <unistd.h>
#include <gdk/gdk.h>
#include <locale.h>


#define TYPE_GUI (gui_get_type ())
#define GUI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_GUI, Gui))
#define GUI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_GUI, GuiClass))
#define IS_GUI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_GUI))
#define IS_GUI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_GUI))
#define GUI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_GUI, GuiClass))

typedef struct _Gui Gui;
typedef struct _GuiClass GuiClass;
typedef struct _GuiPrivate GuiPrivate;

struct _Gui {
	GObject parent_instance;
	GuiPrivate * priv;
};

struct _GuiClass {
	GObjectClass parent_class;
};

struct _GuiPrivate {
	GtkBuilder* builder;
	GtkMenuShell* menu;
	GtkMenu* systraymenu;
	GtkWindow* top;
	GtkWindow* adddialog;
	GtkWindow* aboutdialog;
	GtkStatusIcon* systray;
	GtkAction* systraymenu_restore;
	GtkTreeView* files;
	GtkComboBox* redirection_type;
	GtkEntry* ssh_host;
	GtkEntry* ssh_port;
	GtkEntry* ssh_user;
	GtkEntry* ssh_password;
	GtkButton* redirection_apply;
	GtkStatusbar* statusbar;
	GtkFileChooserButton* localdirectory;
	GtkEntry* shareas;
	GtkTextView* logtext;
	GtkClipboard* clipboard;
	GtkListStore* model;
	char* string_model;
	char* public_url;
	char* preferred_share;
	char* invitation;
	guint lastlog;
	guint pid;
	char* display;
	DBusGProxy* _remote;
};


static gpointer gui_parent_class = NULL;

#define UI_FILENAME "gui.ui"
#define UI_PATH "ui:" DATADIR "/meiga/ui"
GType gui_get_type (void);
#define GUI_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_GUI, GuiPrivate))
enum  {
	GUI_DUMMY_PROPERTY
};
void gui_on_top_hide (GtkWidget* widget, Gui* self);
void gui_on_systray (GtkStatusIcon* widget, Gui* self);
void gui_on_systray_menu (GtkStatusIcon* widget, guint button, guint activateTime, Gui* self);
void gui_on_restore (GtkWidget* widget, Gui* self);
static DBusGProxy* gui_get_remote (Gui* self);
static GList* gui_get_selected_shares (Gui* self);
void _dynamic_unregister_path0 (DBusGProxy* self, const char* param1, GError** error);
static void gui_log (Gui* self, const char* msg);
static void _g_list_free_g_free (GList* self);
void gui_on_remove (GtkWidget* widget, Gui* self);
void gui_on_add (GtkWidget* widget, Gui* self);
void gui_quit (Gui* self);
void gui_on_quit (GtkWidget* widget, Gui* self);
void _dynamic_set_ssh_host1 (DBusGProxy* self, const char* param1, GError** error);
void _dynamic_set_ssh_port2 (DBusGProxy* self, const char* param1, GError** error);
void _dynamic_set_ssh_user3 (DBusGProxy* self, const char* param1, GError** error);
void _dynamic_set_ssh_password4 (DBusGProxy* self, const char* param1, GError** error);
void _dynamic_set_redirection_type5 (DBusGProxy* self, gint param1, GError** error);
void gui_on_redirection_apply (GtkWidget* widget, Gui* self);
void gui_on_adddialogcancel (GtkWidget* widget, Gui* self);
static void gui_error_dialog (Gui* self, const char* msg);
void _dynamic_register_path6 (DBusGProxy* self, const char* param1, const char* param2, GError** error);
void gui_on_adddialogok (GtkWidget* widget, Gui* self);
void gui_on_about (GtkWidget* widget, Gui* self);
void gui_on_aboutdialogclose (GtkWidget* widget, Gui* self);
void gui_on_copy_invitation (GtkWidget* widget, Gui* self);
static void _g_list_free_g_object_unref (GList* self);
static GtkMenuBar* gui_menushell_to_menubar (Gui* self, GtkMenuShell* menu);
void _dynamic_register_gui7 (DBusGProxy* self, guint param1, const char* param2, GError** error);
void gui_on_remote_model_changed (Gui* self);
static void _gui_on_remote_model_changed_dynamic_ModelChanged0_ (DBusGProxy* _sender, gpointer self);
void _dynamic_ModelChanged1_connect (gpointer obj, const char * signal_name, GCallback handler, gpointer data);
void gui_on_remote_log_changed (Gui* self);
static void _gui_on_remote_log_changed_dynamic_LogChanged2_ (DBusGProxy* _sender, gpointer self);
void _dynamic_LogChanged3_connect (gpointer obj, const char * signal_name, GCallback handler, gpointer data);
static void gui_dbus_init (Gui* self);
static void gui_update_model (Gui* self);
static void gui_update_statusbar (Gui* self);
static void gui_nautilus_init (Gui* self);
static void gui_update_log (Gui* self);
static void _gui_on_systray_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self);
static void _gui_on_systray_menu_gtk_status_icon_popup_menu (GtkStatusIcon* _sender, guint button, guint activate_time, gpointer self);
static void gui_gui_init (Gui* self);
static void gui_update_model_from_string (Gui* self, GtkListStore* model, const char* string_model);
char* _dynamic_get_paths_as_string8 (DBusGProxy* self, GError** error);
gint _dynamic_get_redirection_type9 (DBusGProxy* self, GError** error);
gint _dynamic_get_redirection_status10 (DBusGProxy* self, GError** error);
char* _dynamic_get_pending_log11 (DBusGProxy* self, guint param1, GError** error);
gint _dynamic_get_redirection_status12 (DBusGProxy* self, GError** error);
char* _dynamic_get_public_url13 (DBusGProxy* self, GError** error);
char* _dynamic_get_requests_stats14 (DBusGProxy* self, GError** error);
static void _g_list_free_gtk_tree_path_free (GList* self);
void gui_init (Gui* self);
void _dynamic_shutdown15 (DBusGProxy* self, GError** error);
Gui* gui_new (void);
Gui* gui_construct (GType object_type);
gint gui_main (char** args, int args_length1);
static void gui_finalize (GObject* obj);
static void gui_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



/* Callbacks*/
void gui_on_top_hide (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_object_set ((GObject*) self->priv->systraymenu_restore, "sensitive", TRUE, NULL);
}


void gui_on_systray (GtkStatusIcon* widget, Gui* self) {
	gboolean visible;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	visible = FALSE;
	g_object_get ((GObject*) self->priv->top, "visible", &visible, NULL);
	g_object_set ((GObject*) self->priv->top, "visible", !visible, NULL);
	g_object_set ((GObject*) self->priv->systraymenu_restore, "sensitive", visible, NULL);
}


void gui_on_systray_menu (GtkStatusIcon* widget, guint button, guint activateTime, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	if (button != 3) {
		gui_on_systray (widget, self);
		return;
	}
	gtk_menu_popup (self->priv->systraymenu, NULL, NULL, NULL, NULL, (guint) 0, (guint32) activateTime);
}


void gui_on_restore (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_object_set ((GObject*) self->priv->top, "visible", TRUE, NULL);
	g_object_set ((GObject*) self->priv->systraymenu_restore, "sensitive", FALSE, NULL);
}


void _dynamic_unregister_path0 (DBusGProxy* self, const char* param1, GError** error) {
	dbus_g_proxy_call (self, "UnregisterPath", error, G_TYPE_STRING, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


void gui_on_remove (GtkWidget* widget, Gui* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_inner_error_ = NULL;
	if (gui_get_remote (self) == NULL) {
		return;
	}
	{
		GList* share_collection;
		GList* share_it;
		share_collection = gui_get_selected_shares (self);
		for (share_it = share_collection; share_it != NULL; share_it = share_it->next) {
			const char* _tmp1_;
			char* share;
			_tmp1_ = NULL;
			share = (_tmp1_ = (const char*) share_it->data, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_));
			{
				{
					_dynamic_unregister_path0 (gui_get_remote (self), share, &_inner_error_);
					if (_inner_error_ != NULL) {
						goto __catch0_g_error;
						goto __finally0;
					}
				}
				goto __finally0;
				__catch0_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
						char* _tmp0_;
						_tmp0_ = NULL;
						gui_log (self, _tmp0_ = g_strdup_printf (_ ("Remote error deleting share '%s'\n"), share));
						_tmp0_ = (g_free (_tmp0_), NULL);
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
					}
				}
				__finally0:
				if (_inner_error_ != NULL) {
					share = (g_free (share), NULL);
					(share_collection == NULL) ? NULL : (share_collection = (_g_list_free_g_free (share_collection), NULL));
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				share = (g_free (share), NULL);
			}
		}
		(share_collection == NULL) ? NULL : (share_collection = (_g_list_free_g_free (share_collection), NULL));
	}
}


void gui_on_add (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	if (gui_get_remote (self) == NULL) {
		return;
	}
	g_object_set ((GObject*) self->priv->adddialog, "visible", TRUE, NULL);
}


void gui_on_quit (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	gui_quit (self);
}


void _dynamic_set_ssh_host1 (DBusGProxy* self, const char* param1, GError** error) {
	dbus_g_proxy_call (self, "SetSshHost", error, G_TYPE_STRING, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void _dynamic_set_ssh_port2 (DBusGProxy* self, const char* param1, GError** error) {
	dbus_g_proxy_call (self, "SetSshPort", error, G_TYPE_STRING, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void _dynamic_set_ssh_user3 (DBusGProxy* self, const char* param1, GError** error) {
	dbus_g_proxy_call (self, "SetSshUser", error, G_TYPE_STRING, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void _dynamic_set_ssh_password4 (DBusGProxy* self, const char* param1, GError** error) {
	dbus_g_proxy_call (self, "SetSshPassword", error, G_TYPE_STRING, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void _dynamic_set_redirection_type5 (DBusGProxy* self, gint param1, GError** error) {
	dbus_g_proxy_call (self, "SetRedirectionType", error, G_TYPE_INT, param1, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void gui_on_redirection_apply (GtkWidget* widget, Gui* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_inner_error_ = NULL;
	if (gui_get_remote (self) == NULL) {
		return;
	}
	{
		_dynamic_set_ssh_host1 (gui_get_remote (self), gtk_entry_get_text (self->priv->ssh_host), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		_dynamic_set_ssh_port2 (gui_get_remote (self), gtk_entry_get_text (self->priv->ssh_port), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		_dynamic_set_ssh_user3 (gui_get_remote (self), gtk_entry_get_text (self->priv->ssh_user), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		_dynamic_set_ssh_password4 (gui_get_remote (self), gtk_entry_get_text (self->priv->ssh_password), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		_dynamic_set_redirection_type5 (gui_get_remote (self), gtk_combo_box_get_active (self->priv->redirection_type), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			gui_log (self, _ ("Remote error applying redirection options\n"));
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


void gui_on_adddialogcancel (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_object_set ((GObject*) self->priv->adddialog, "visible", FALSE, NULL);
	/* Clear data for next use*/
	gtk_entry_set_text (self->priv->shareas, "");
}


static char* string_replace (const char* self, const char* old, const char* replacement) {
	char* result;
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
		_tmp0_ = NULL;
		_tmp1_ = NULL;
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch2_g_regex_error;
			}
			goto __finally2;
		}
		result = _tmp2_;
		(regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL));
		return result;
	}
	goto __finally2;
	__catch2_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


void _dynamic_register_path6 (DBusGProxy* self, const char* param1, const char* param2, GError** error) {
	dbus_g_proxy_call (self, "RegisterPath", error, G_TYPE_STRING, param1, G_TYPE_STRING, param2, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void gui_on_adddialogok (GtkWidget* widget, Gui* self) {
	GError * _inner_error_;
	char* local_file;
	const char* _tmp0_;
	char* shared_as;
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	_inner_error_ = NULL;
	local_file = gtk_file_chooser_get_filename ((GtkFileChooser*) self->priv->localdirectory);
	_tmp0_ = NULL;
	shared_as = (_tmp0_ = gtk_entry_get_text (self->priv->shareas), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	if (shared_as != NULL) {
		char* _tmp2_;
		gboolean _tmp3_;
		if (g_utf8_get_char (g_utf8_offset_to_pointer (shared_as, 0)) != '/') {
			char* _tmp1_;
			_tmp1_ = NULL;
			shared_as = (_tmp1_ = g_strconcat ("/", shared_as, NULL), shared_as = (g_free (shared_as), NULL), _tmp1_);
		}
		_tmp2_ = NULL;
		if ((_tmp3_ = strcmp (_tmp2_ = string_replace (shared_as, "/", ""), "") == 0, _tmp2_ = (g_free (_tmp2_), NULL), _tmp3_)) {
			gui_error_dialog (self, _ ("Empty share name not allowed"));
			local_file = (g_free (local_file), NULL);
			shared_as = (g_free (shared_as), NULL);
			return;
		}
		if (strcmp (shared_as, "/rss") == 0) {
			gui_error_dialog (self, _ ("Share name not allowed"));
			local_file = (g_free (local_file), NULL);
			shared_as = (g_free (shared_as), NULL);
			return;
		}
	}
	if (gui_get_remote (self) != NULL) {
		{
			_dynamic_register_path6 (gui_get_remote (self), local_file, shared_as, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch3_g_error;
				goto __finally3;
			}
		}
		goto __finally3;
		__catch3_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				char* _tmp4_;
				_tmp4_ = NULL;
				gui_log (self, _tmp4_ = g_strdup_printf (_ ("Remote error sharing '%s' as '%s'\n"), local_file, shared_as));
				_tmp4_ = (g_free (_tmp4_), NULL);
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally3:
		if (_inner_error_ != NULL) {
			local_file = (g_free (local_file), NULL);
			shared_as = (g_free (shared_as), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	/* Clear data for next use*/
	gtk_entry_set_text (self->priv->shareas, "");
	g_object_set ((GObject*) self->priv->adddialog, "visible", FALSE, NULL);
	local_file = (g_free (local_file), NULL);
	shared_as = (g_free (shared_as), NULL);
}


void gui_on_about (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_object_set ((GObject*) self->priv->aboutdialog, "visible", TRUE, NULL);
}


void gui_on_aboutdialogclose (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	g_object_set ((GObject*) self->priv->aboutdialog, "visible", FALSE, NULL);
}


void gui_on_copy_invitation (GtkWidget* widget, Gui* self) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (widget != NULL);
	gtk_clipboard_set_text (self->priv->clipboard, self->priv->invitation, -1);
}


static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


/* Private methods*/
static GtkMenuBar* gui_menushell_to_menubar (Gui* self, GtkMenuShell* menu) {
	GtkMenuBar* result;
	GtkMenuBar* menubar;
	GList* children;
	GList* childrencopy;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (menu != NULL, NULL);
	menubar = g_object_ref_sink ((GtkMenuBar*) gtk_menu_bar_new ());
	children = (GList*) menu->children;
	childrencopy = NULL;
	{
		GList* menuitem_collection;
		GList* menuitem_it;
		menuitem_collection = children;
		for (menuitem_it = menuitem_collection; menuitem_it != NULL; menuitem_it = menuitem_it->next) {
			GtkMenuItem* _tmp1_;
			GtkMenuItem* menuitem;
			_tmp1_ = NULL;
			menuitem = (_tmp1_ = (GtkMenuItem*) menuitem_it->data, (_tmp1_ == NULL) ? NULL : g_object_ref (_tmp1_));
			{
				GtkMenuItem* _tmp0_;
				_tmp0_ = NULL;
				childrencopy = g_list_append (childrencopy, (_tmp0_ = menuitem, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)));
				(menuitem == NULL) ? NULL : (menuitem = (g_object_unref (menuitem), NULL));
			}
		}
	}
	{
		GList* menuitem_collection;
		GList* menuitem_it;
		menuitem_collection = childrencopy;
		for (menuitem_it = menuitem_collection; menuitem_it != NULL; menuitem_it = menuitem_it->next) {
			GtkMenuItem* _tmp2_;
			GtkMenuItem* menuitem;
			_tmp2_ = NULL;
			menuitem = (_tmp2_ = (GtkMenuItem*) menuitem_it->data, (_tmp2_ == NULL) ? NULL : g_object_ref (_tmp2_));
			{
				gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent ((GtkWidget*) menuitem)), GTK_WIDGET (menuitem));
				gtk_menu_shell_append ((GtkMenuShell*) menubar, (GtkWidget*) menuitem);
				(menuitem == NULL) ? NULL : (menuitem = (g_object_unref (menuitem), NULL));
			}
		}
	}
	result = menubar;
	(childrencopy == NULL) ? NULL : (childrencopy = (_g_list_free_g_object_unref (childrencopy), NULL));
	return result;
}


static void gui_error_dialog (Gui* self, const char* msg) {
	GtkMessageDialog* d;
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	d = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (self->priv->adddialog, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "%s", msg, NULL));
	gtk_dialog_run ((GtkDialog*) d);
	gtk_object_destroy ((GtkObject*) d);
	(d == NULL) ? NULL : (d = (g_object_unref (d), NULL));
}


static void gui_log (Gui* self, const char* msg) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	if (self->priv->logtext != NULL) {
		GtkTextBuffer* _tmp0_;
		GtkTextBuffer* b;
		GtkTextIter end = {0};
		_tmp0_ = NULL;
		b = (_tmp0_ = gtk_text_view_get_buffer (self->priv->logtext), (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_));
		gtk_text_buffer_get_end_iter (b, &end);
		gtk_text_buffer_insert (gtk_text_view_get_buffer (self->priv->logtext), &end, msg, -1);
		(b == NULL) ? NULL : (b = (g_object_unref (b), NULL));
	} else {
		fprintf (stderr, "%s", msg);
	}
}


void _dynamic_register_gui7 (DBusGProxy* self, guint param1, const char* param2, GError** error) {
	dbus_g_proxy_call (self, "RegisterGui", error, G_TYPE_UINT, param1, G_TYPE_STRING, param2, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


static void _gui_on_remote_model_changed_dynamic_ModelChanged0_ (DBusGProxy* _sender, gpointer self) {
	gui_on_remote_model_changed (self);
}


void _dynamic_ModelChanged1_connect (gpointer obj, const char * signal_name, GCallback handler, gpointer data) {
	dbus_g_object_register_marshaller (g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, G_TYPE_INVALID);
	dbus_g_proxy_add_signal (obj, "ModelChanged", G_TYPE_INVALID);
	dbus_g_proxy_connect_signal (obj, signal_name, handler, data, NULL);
}


static void _gui_on_remote_log_changed_dynamic_LogChanged2_ (DBusGProxy* _sender, gpointer self) {
	gui_on_remote_log_changed (self);
}


void _dynamic_LogChanged3_connect (gpointer obj, const char * signal_name, GCallback handler, gpointer data) {
	dbus_g_object_register_marshaller (g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, G_TYPE_INVALID);
	dbus_g_proxy_add_signal (obj, "LogChanged", G_TYPE_INVALID);
	dbus_g_proxy_connect_signal (obj, signal_name, handler, data, NULL);
}


static void gui_dbus_init (Gui* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	{
		DBusGConnection* conn;
		DBusGProxy* _tmp0_;
		conn = dbus_g_bus_get (DBUS_BUS_SESSION, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch4_g_error;
			goto __finally4;
		}
		_tmp0_ = NULL;
		self->priv->_remote = (_tmp0_ = dbus_g_proxy_new_for_name (conn, "com.igalia.Meiga", "/com/igalia/Meiga", "com.igalia.Meiga"), (self->priv->_remote == NULL) ? NULL : (self->priv->_remote = (g_object_unref (self->priv->_remote), NULL)), _tmp0_);
		(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
	}
	goto __finally4;
	__catch4_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
			_tmp1_ = NULL;
			gui_log (self, _tmp1_ = g_strdup_printf (_ ("Error looking for DBUS server: %s\n"), e->message));
			_tmp1_ = (g_free (_tmp1_), NULL);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally4:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	{
		/* Test the connection*/
		if (self->priv->_remote != NULL) {
			_dynamic_register_gui7 (self->priv->_remote, self->priv->pid, self->priv->display, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch5_g_error;
				goto __finally5;
			}
		}
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			DBusGProxy* _tmp2_;
			gui_log (self, _ ("Error looking for DBUS server: remote object not found\n"));
			_tmp2_ = NULL;
			self->priv->_remote = (_tmp2_ = NULL, (self->priv->_remote == NULL) ? NULL : (self->priv->_remote = (g_object_unref (self->priv->_remote), NULL)), _tmp2_);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally5:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	/* Attach to remote signals*/
	if (self->priv->_remote != NULL) {
		_dynamic_ModelChanged1_connect (self->priv->_remote, "ModelChanged", (GCallback) _gui_on_remote_model_changed_dynamic_ModelChanged0_, self);
		_dynamic_LogChanged3_connect (self->priv->_remote, "LogChanged", (GCallback) _gui_on_remote_log_changed_dynamic_LogChanged2_, self);
	}
}


void gui_on_remote_model_changed (Gui* self) {
	g_return_if_fail (self != NULL);
	gui_update_model (self);
	gui_update_statusbar (self);
}


static void gui_nautilus_init (Gui* self) {
	char* menudir;
	char* menulink;
	char* menufile;
	g_return_if_fail (self != NULL);
	/* If there's no link to the Nautilus menu entry in the
	 user's directory, create it*/
	menudir = g_strconcat (g_get_home_dir (), "/.gnome2/nautilus-scripts/", NULL);
	menulink = g_strconcat (menudir, _ ("Share on Meiga..."), NULL);
	menufile = g_strdup (DATADIR "/nautilus-scripts/" "share-on-meiga");
	if (!g_file_test (menulink, G_FILE_TEST_EXISTS)) {
		char* _tmp0_;
		_tmp0_ = NULL;
		gui_log (self, _tmp0_ = g_strdup_printf (_ ("Creating Nautilus context menu for Meiga %s --> %s\n"), menulink, menufile));
		_tmp0_ = (g_free (_tmp0_), NULL);
		g_mkdir_with_parents (menudir, 0700);
		symlink (menufile, menulink);
	}
	menudir = (g_free (menudir), NULL);
	menulink = (g_free (menulink), NULL);
	menufile = (g_free (menufile), NULL);
}


static void _gui_on_systray_gtk_status_icon_activate (GtkStatusIcon* _sender, gpointer self) {
	gui_on_systray (_sender, self);
}


static void _gui_on_systray_menu_gtk_status_icon_popup_menu (GtkStatusIcon* _sender, guint button, guint activate_time, gpointer self) {
	gui_on_systray_menu (_sender, button, activate_time, self);
}


static void gui_gui_init (Gui* self) {
	GError * _inner_error_;
	char** _tmp1_;
	gint path_size;
	gint path_length1;
	char** _tmp0_;
	char** path;
	char* iconfile;
	gboolean gui_loaded;
	GtkBuilder* _tmp2_;
	GtkMenu* _tmp10_;
	GtkMenu* _tmp9_;
	GtkAction* _tmp12_;
	GtkAction* _tmp11_;
	GtkWindow* _tmp14_;
	GtkWindow* _tmp13_;
	GtkWindow* _tmp16_;
	GtkWindow* _tmp15_;
	GtkWindow* _tmp18_;
	GtkWindow* _tmp17_;
	GtkTreeView* _tmp20_;
	GtkTreeView* _tmp19_;
	GtkComboBox* _tmp22_;
	GtkComboBox* _tmp21_;
	GtkEntry* _tmp24_;
	GtkEntry* _tmp23_;
	GtkEntry* _tmp26_;
	GtkEntry* _tmp25_;
	GtkEntry* _tmp28_;
	GtkEntry* _tmp27_;
	GtkEntry* _tmp30_;
	GtkEntry* _tmp29_;
	GtkEntry* _tmp32_;
	GtkEntry* _tmp31_;
	GtkButton* _tmp34_;
	GtkButton* _tmp33_;
	GtkStatusbar* _tmp36_;
	GtkStatusbar* _tmp35_;
	GtkFileChooserButton* _tmp38_;
	GtkFileChooserButton* _tmp37_;
	GtkEntry* _tmp40_;
	GtkEntry* _tmp39_;
	GtkTextView* _tmp42_;
	GtkTextView* _tmp41_;
	GtkVBox* _tmp43_;
	GtkVBox* topvbox;
	GtkListStore* redirection_type_model;
	GtkTreeIter redirection_type_model_iter = {0};
	GtkCellRenderer* redirection_type_model_cell_renderer;
	GtkCellRenderer* _tmp44_;
	GtkTextView* _tmp45_;
	GtkTextView* abouttext;
	GtkMenuShell* _tmp46_;
	GtkListStore* _tmp47_;
	GtkCellRendererText* _tmp48_;
	GtkCellRendererText* _tmp49_;
	char* _tmp51_;
	const char* _tmp50_;
	char* _tmp52_;
	GtkClipboard* _tmp54_;
	GtkClipboard* _tmp53_;
	GtkStatusIcon* _tmp55_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	path = (_tmp1_ = _tmp0_ = g_strsplit (UI_PATH, ":", 8), path_length1 = _vala_array_length (_tmp0_), path_size = path_length1, _tmp1_);
	iconfile = NULL;
	gui_loaded = FALSE;
	_tmp2_ = NULL;
	self->priv->builder = (_tmp2_ = gtk_builder_new (), (self->priv->builder == NULL) ? NULL : (self->priv->builder = (g_object_unref (self->priv->builder), NULL)), _tmp2_);
	{
		gint i;
		i = 0;
		{
			gboolean _tmp3_;
			_tmp3_ = TRUE;
			while (TRUE) {
				char* _tmp4_;
				char* _tmp5_;
				char* filename;
				if (!_tmp3_) {
					i++;
				}
				_tmp3_ = FALSE;
				if (!(path[i] != NULL)) {
					break;
				}
				_tmp4_ = NULL;
				_tmp5_ = NULL;
				filename = (_tmp5_ = g_strconcat (_tmp4_ = g_strconcat (path[i], "/", NULL), UI_FILENAME, NULL), _tmp4_ = (g_free (_tmp4_), NULL), _tmp5_);
				{
					char* _tmp7_;
					char* _tmp6_;
					gtk_builder_set_translation_domain (self->priv->builder, "meiga");
					gtk_builder_add_from_file (self->priv->builder, filename, &_inner_error_);
					if (_inner_error_ != NULL) {
						goto __catch6_g_error;
						goto __finally6;
					}
					gui_loaded = TRUE;
					_tmp7_ = NULL;
					_tmp6_ = NULL;
					iconfile = (_tmp7_ = g_strconcat (_tmp6_ = g_strconcat (path[i], "/", NULL), "meiga-16x16.png", NULL), iconfile = (g_free (iconfile), NULL), _tmp7_);
					_tmp6_ = (g_free (_tmp6_), NULL);
					filename = (g_free (filename), NULL);
					break;
				}
				goto __finally6;
				__catch6_g_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						filename = (g_free (filename), NULL);
						continue;
					}
				}
				__finally6:
				if (_inner_error_ != NULL) {
					filename = (g_free (filename), NULL);
					path = (_vala_array_free (path, path_length1, (GDestroyNotify) g_free), NULL);
					iconfile = (g_free (iconfile), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				filename = (g_free (filename), NULL);
			}
		}
	}
	if (!gui_loaded) {
		char* _tmp8_;
		_tmp8_ = NULL;
		gui_log (self, _tmp8_ = g_strdup_printf (_ ("Could not load UI file %s\n"), UI_FILENAME));
		_tmp8_ = (g_free (_tmp8_), NULL);
		gui_quit (self);
	}
	gtk_builder_connect_signals (self->priv->builder, self);
	_tmp10_ = NULL;
	_tmp9_ = NULL;
	self->priv->systraymenu = (_tmp10_ = (_tmp9_ = GTK_MENU (gtk_builder_get_object (self->priv->builder, "systraymenu")), (_tmp9_ == NULL) ? NULL : g_object_ref (_tmp9_)), (self->priv->systraymenu == NULL) ? NULL : (self->priv->systraymenu = (g_object_unref (self->priv->systraymenu), NULL)), _tmp10_);
	_tmp12_ = NULL;
	_tmp11_ = NULL;
	self->priv->systraymenu_restore = (_tmp12_ = (_tmp11_ = GTK_ACTION (gtk_builder_get_object (self->priv->builder, "tray_restore")), (_tmp11_ == NULL) ? NULL : g_object_ref (_tmp11_)), (self->priv->systraymenu_restore == NULL) ? NULL : (self->priv->systraymenu_restore = (g_object_unref (self->priv->systraymenu_restore), NULL)), _tmp12_);
	_tmp14_ = NULL;
	_tmp13_ = NULL;
	self->priv->top = (_tmp14_ = (_tmp13_ = GTK_WINDOW (gtk_builder_get_object (self->priv->builder, "top")), (_tmp13_ == NULL) ? NULL : g_object_ref (_tmp13_)), (self->priv->top == NULL) ? NULL : (self->priv->top = (g_object_unref (self->priv->top), NULL)), _tmp14_);
	_tmp16_ = NULL;
	_tmp15_ = NULL;
	self->priv->adddialog = (_tmp16_ = (_tmp15_ = GTK_WINDOW (gtk_builder_get_object (self->priv->builder, "adddialog")), (_tmp15_ == NULL) ? NULL : g_object_ref (_tmp15_)), (self->priv->adddialog == NULL) ? NULL : (self->priv->adddialog = (g_object_unref (self->priv->adddialog), NULL)), _tmp16_);
	_tmp18_ = NULL;
	_tmp17_ = NULL;
	self->priv->aboutdialog = (_tmp18_ = (_tmp17_ = GTK_WINDOW (gtk_builder_get_object (self->priv->builder, "aboutdialog")), (_tmp17_ == NULL) ? NULL : g_object_ref (_tmp17_)), (self->priv->aboutdialog == NULL) ? NULL : (self->priv->aboutdialog = (g_object_unref (self->priv->aboutdialog), NULL)), _tmp18_);
	_tmp20_ = NULL;
	_tmp19_ = NULL;
	self->priv->files = (_tmp20_ = (_tmp19_ = GTK_TREE_VIEW (gtk_builder_get_object (self->priv->builder, "files")), (_tmp19_ == NULL) ? NULL : g_object_ref (_tmp19_)), (self->priv->files == NULL) ? NULL : (self->priv->files = (g_object_unref (self->priv->files), NULL)), _tmp20_);
	_tmp22_ = NULL;
	_tmp21_ = NULL;
	self->priv->redirection_type = (_tmp22_ = (_tmp21_ = GTK_COMBO_BOX (gtk_builder_get_object (self->priv->builder, "cbredirection_type")), (_tmp21_ == NULL) ? NULL : g_object_ref (_tmp21_)), (self->priv->redirection_type == NULL) ? NULL : (self->priv->redirection_type = (g_object_unref (self->priv->redirection_type), NULL)), _tmp22_);
	_tmp24_ = NULL;
	_tmp23_ = NULL;
	self->priv->ssh_host = (_tmp24_ = (_tmp23_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "essh_host")), (_tmp23_ == NULL) ? NULL : g_object_ref (_tmp23_)), (self->priv->ssh_host == NULL) ? NULL : (self->priv->ssh_host = (g_object_unref (self->priv->ssh_host), NULL)), _tmp24_);
	_tmp26_ = NULL;
	_tmp25_ = NULL;
	self->priv->ssh_port = (_tmp26_ = (_tmp25_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "essh_port")), (_tmp25_ == NULL) ? NULL : g_object_ref (_tmp25_)), (self->priv->ssh_port == NULL) ? NULL : (self->priv->ssh_port = (g_object_unref (self->priv->ssh_port), NULL)), _tmp26_);
	_tmp28_ = NULL;
	_tmp27_ = NULL;
	self->priv->ssh_user = (_tmp28_ = (_tmp27_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "essh_user")), (_tmp27_ == NULL) ? NULL : g_object_ref (_tmp27_)), (self->priv->ssh_user == NULL) ? NULL : (self->priv->ssh_user = (g_object_unref (self->priv->ssh_user), NULL)), _tmp28_);
	_tmp30_ = NULL;
	_tmp29_ = NULL;
	self->priv->ssh_password = (_tmp30_ = (_tmp29_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "essh_password")), (_tmp29_ == NULL) ? NULL : g_object_ref (_tmp29_)), (self->priv->ssh_password == NULL) ? NULL : (self->priv->ssh_password = (g_object_unref (self->priv->ssh_password), NULL)), _tmp30_);
	_tmp32_ = NULL;
	_tmp31_ = NULL;
	self->priv->ssh_user = (_tmp32_ = (_tmp31_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "essh_user")), (_tmp31_ == NULL) ? NULL : g_object_ref (_tmp31_)), (self->priv->ssh_user == NULL) ? NULL : (self->priv->ssh_user = (g_object_unref (self->priv->ssh_user), NULL)), _tmp32_);
	_tmp34_ = NULL;
	_tmp33_ = NULL;
	self->priv->redirection_apply = (_tmp34_ = (_tmp33_ = GTK_BUTTON (gtk_builder_get_object (self->priv->builder, "bredirection_apply")), (_tmp33_ == NULL) ? NULL : g_object_ref (_tmp33_)), (self->priv->redirection_apply == NULL) ? NULL : (self->priv->redirection_apply = (g_object_unref (self->priv->redirection_apply), NULL)), _tmp34_);
	_tmp36_ = NULL;
	_tmp35_ = NULL;
	self->priv->statusbar = (_tmp36_ = (_tmp35_ = GTK_STATUSBAR (gtk_builder_get_object (self->priv->builder, "statusbar")), (_tmp35_ == NULL) ? NULL : g_object_ref (_tmp35_)), (self->priv->statusbar == NULL) ? NULL : (self->priv->statusbar = (g_object_unref (self->priv->statusbar), NULL)), _tmp36_);
	_tmp38_ = NULL;
	_tmp37_ = NULL;
	self->priv->localdirectory = (_tmp38_ = (_tmp37_ = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (self->priv->builder, "localdirectory")), (_tmp37_ == NULL) ? NULL : g_object_ref (_tmp37_)), (self->priv->localdirectory == NULL) ? NULL : (self->priv->localdirectory = (g_object_unref (self->priv->localdirectory), NULL)), _tmp38_);
	_tmp40_ = NULL;
	_tmp39_ = NULL;
	self->priv->shareas = (_tmp40_ = (_tmp39_ = GTK_ENTRY (gtk_builder_get_object (self->priv->builder, "shareas")), (_tmp39_ == NULL) ? NULL : g_object_ref (_tmp39_)), (self->priv->shareas == NULL) ? NULL : (self->priv->shareas = (g_object_unref (self->priv->shareas), NULL)), _tmp40_);
	_tmp42_ = NULL;
	_tmp41_ = NULL;
	self->priv->logtext = (_tmp42_ = (_tmp41_ = GTK_TEXT_VIEW (gtk_builder_get_object (self->priv->builder, "logtext")), (_tmp41_ == NULL) ? NULL : g_object_ref (_tmp41_)), (self->priv->logtext == NULL) ? NULL : (self->priv->logtext = (g_object_unref (self->priv->logtext), NULL)), _tmp42_);
	_tmp43_ = NULL;
	topvbox = (_tmp43_ = GTK_VBOX (gtk_builder_get_object (self->priv->builder, "topvbox")), (_tmp43_ == NULL) ? NULL : g_object_ref (_tmp43_));
	{
		gtk_window_set_icon_from_file (self->priv->top, iconfile, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch7_g_error;
			goto __finally7;
		}
		gtk_window_set_icon_from_file (self->priv->adddialog, iconfile, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch7_g_error;
			goto __finally7;
		}
		gtk_window_set_icon_from_file (self->priv->aboutdialog, iconfile, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch7_g_error;
			goto __finally7;
		}
	}
	goto __finally7;
	__catch7_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			gui_log (self, _ ("Icon file not found\n"));
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally7:
	if (_inner_error_ != NULL) {
		path = (_vala_array_free (path, path_length1, (GDestroyNotify) g_free), NULL);
		iconfile = (g_free (iconfile), NULL);
		(topvbox == NULL) ? NULL : (topvbox = (g_object_unref (topvbox), NULL));
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	redirection_type_model = gtk_list_store_new (1, G_TYPE_STRING, NULL);
	redirection_type_model_cell_renderer = NULL;
	gtk_combo_box_set_model (self->priv->redirection_type, (GtkTreeModel*) redirection_type_model);
	gtk_list_store_append (redirection_type_model, &redirection_type_model_iter);
	gtk_list_store_set (redirection_type_model, &redirection_type_model_iter, 0, _ ("None"), -1, -1);
	gtk_list_store_append (redirection_type_model, &redirection_type_model_iter);
	gtk_list_store_set (redirection_type_model, &redirection_type_model_iter, 0, _ ("UPnP"), -1, -1);
	gtk_list_store_append (redirection_type_model, &redirection_type_model_iter);
	gtk_list_store_set (redirection_type_model, &redirection_type_model_iter, 0, _ ("SSH"), -1, -1);
	_tmp44_ = NULL;
	redirection_type_model_cell_renderer = (_tmp44_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), (redirection_type_model_cell_renderer == NULL) ? NULL : (redirection_type_model_cell_renderer = (g_object_unref (redirection_type_model_cell_renderer), NULL)), _tmp44_);
	gtk_cell_layout_pack_start ((GtkCellLayout*) self->priv->redirection_type, redirection_type_model_cell_renderer, TRUE);
	gtk_cell_layout_set_attributes ((GtkCellLayout*) self->priv->redirection_type, redirection_type_model_cell_renderer, "text", 0, NULL);
	_tmp45_ = NULL;
	abouttext = (_tmp45_ = GTK_TEXT_VIEW (gtk_builder_get_object (self->priv->builder, "abouttext")), (_tmp45_ == NULL) ? NULL : g_object_ref (_tmp45_));
	gtk_text_buffer_insert_at_cursor (gtk_text_view_get_buffer (abouttext), _ ("Meiga - Copyright (C) 2009 Igalia, S.L.\n" "\n" "http://meiga.igalia.com\n" "\n" "This program comes with ABSOLUTELY NO WARRANTY.\n" "Licensed under GNU GPL 2.0. This is free software, and you are welcome to " "redistribute it under certain conditions.\n" "\n" "For more information, see:\n" "\n" "http://www.gnu.org/licenses/old-licenses/gpl-2.0.html"), -1);
	_tmp46_ = NULL;
	self->priv->menu = (_tmp46_ = (GtkMenuShell*) gui_menushell_to_menubar (self, (GtkMenuShell*) GTK_MENU (gtk_builder_get_object (self->priv->builder, "menu"))), (self->priv->menu == NULL) ? NULL : (self->priv->menu = (g_object_unref (self->priv->menu), NULL)), _tmp46_);
	g_object_set ((GObject*) self->priv->menu, "visible", TRUE, NULL);
	gtk_box_pack_start ((GtkBox*) topvbox, (GtkWidget*) self->priv->menu, FALSE, FALSE, (guint) 0);
	gtk_box_reorder_child ((GtkBox*) topvbox, (GtkWidget*) self->priv->menu, 0);
	_tmp47_ = NULL;
	self->priv->model = (_tmp47_ = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING, NULL), (self->priv->model == NULL) ? NULL : (self->priv->model = (g_object_unref (self->priv->model), NULL)), _tmp47_);
	gtk_tree_view_set_model (self->priv->files, (GtkTreeModel*) self->priv->model);
	_tmp48_ = NULL;
	gtk_tree_view_insert_column_with_attributes (self->priv->files, -1, _ ("Local file"), (GtkCellRenderer*) (_tmp48_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ())), "text", 0, NULL, NULL);
	(_tmp48_ == NULL) ? NULL : (_tmp48_ = (g_object_unref (_tmp48_), NULL));
	_tmp49_ = NULL;
	gtk_tree_view_insert_column_with_attributes (self->priv->files, -1, _ ("Shared as"), (GtkCellRenderer*) (_tmp49_ = g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ())), "text", 1, NULL, NULL);
	(_tmp49_ == NULL) ? NULL : (_tmp49_ = (g_object_unref (_tmp49_), NULL));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (self->priv->files), GTK_SELECTION_MULTIPLE);
	self->priv->pid = (guint) getpid ();
	_tmp51_ = NULL;
	_tmp50_ = NULL;
	self->priv->display = (_tmp51_ = (_tmp50_ = g_getenv ("DISPLAY"), (_tmp50_ == NULL) ? NULL : g_strdup (_tmp50_)), self->priv->display = (g_free (self->priv->display), NULL), _tmp51_);
	_tmp52_ = NULL;
	self->priv->public_url = (_tmp52_ = g_strdup (""), self->priv->public_url = (g_free (self->priv->public_url), NULL), _tmp52_);
	self->priv->lastlog = (guint) 0;
	_tmp54_ = NULL;
	_tmp53_ = NULL;
	self->priv->clipboard = (_tmp54_ = (_tmp53_ = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), (_tmp53_ == NULL) ? NULL : g_object_ref (_tmp53_)), (self->priv->clipboard == NULL) ? NULL : (self->priv->clipboard = (g_object_unref (self->priv->clipboard), NULL)), _tmp54_);
	gui_update_model (self);
	gui_update_log (self);
	_tmp55_ = NULL;
	self->priv->systray = (_tmp55_ = gtk_status_icon_new_from_file (iconfile), (self->priv->systray == NULL) ? NULL : (self->priv->systray = (g_object_unref (self->priv->systray), NULL)), _tmp55_);
	g_signal_connect_object (self->priv->systray, "activate", (GCallback) _gui_on_systray_gtk_status_icon_activate, self, 0);
	g_signal_connect_object (self->priv->systray, "popup-menu", (GCallback) _gui_on_systray_menu_gtk_status_icon_popup_menu, self, 0);
	/* App is finally shown*/
	g_object_set ((GObject*) self->priv->systray, "visible", TRUE, NULL);
	gui_on_restore ((GtkWidget*) self->priv->top, self);
	path = (_vala_array_free (path, path_length1, (GDestroyNotify) g_free), NULL);
	iconfile = (g_free (iconfile), NULL);
	(topvbox == NULL) ? NULL : (topvbox = (g_object_unref (topvbox), NULL));
	(redirection_type_model == NULL) ? NULL : (redirection_type_model = (g_object_unref (redirection_type_model), NULL));
	(redirection_type_model_cell_renderer == NULL) ? NULL : (redirection_type_model_cell_renderer = (g_object_unref (redirection_type_model_cell_renderer), NULL));
	(abouttext == NULL) ? NULL : (abouttext = (g_object_unref (abouttext), NULL));
}


static void gui_update_model_from_string (Gui* self, GtkListStore* model, const char* string_model) {
	char* _tmp0_;
	gboolean _tmp1_;
	char** _tmp3_;
	gint rows_size;
	gint rows_length1;
	char** _tmp2_;
	char** rows;
	g_return_if_fail (self != NULL);
	g_return_if_fail (model != NULL);
	g_return_if_fail (string_model != NULL);
	gtk_list_store_clear (model);
	_tmp0_ = NULL;
	self->priv->preferred_share = (_tmp0_ = NULL, self->priv->preferred_share = (g_free (self->priv->preferred_share), NULL), _tmp0_);
	_tmp1_ = FALSE;
	if (string_model == NULL) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = g_utf8_get_char (g_utf8_offset_to_pointer (string_model, 0)) == '\0';
	}
	if (_tmp1_) {
		return;
	}
	_tmp3_ = NULL;
	_tmp2_ = NULL;
	rows = (_tmp3_ = _tmp2_ = g_strsplit (string_model, "\n", 128), rows_length1 = _vala_array_length (_tmp2_), rows_size = rows_length1, _tmp3_);
	{
		gint i;
		i = 0;
		{
			gboolean _tmp4_;
			_tmp4_ = TRUE;
			while (TRUE) {
				char** _tmp6_;
				gint cols_size;
				gint cols_length1;
				char** _tmp5_;
				char** cols;
				const char* _tmp7_;
				char* local_file;
				const char* _tmp8_;
				char* shared_as;
				GtkTreeIter iter = {0};
				if (!_tmp4_) {
					i++;
				}
				_tmp4_ = FALSE;
				if (!(g_utf8_get_char (g_utf8_offset_to_pointer (rows[i], 0)) != '\0')) {
					break;
				}
				_tmp6_ = NULL;
				_tmp5_ = NULL;
				cols = (_tmp6_ = _tmp5_ = g_strsplit (rows[i], "\t", 2), cols_length1 = _vala_array_length (_tmp5_), cols_size = cols_length1, _tmp6_);
				_tmp7_ = NULL;
				local_file = (_tmp7_ = cols[1], (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
				_tmp8_ = NULL;
				shared_as = (_tmp8_ = cols[0], (_tmp8_ == NULL) ? NULL : g_strdup (_tmp8_));
				gtk_list_store_append (model, &iter);
				gtk_list_store_set (model, &iter, 0, local_file, 1, shared_as, -1);
				if (i == 0) {
					char* _tmp10_;
					const char* _tmp9_;
					_tmp10_ = NULL;
					_tmp9_ = NULL;
					self->priv->preferred_share = (_tmp10_ = (_tmp9_ = shared_as, (_tmp9_ == NULL) ? NULL : g_strdup (_tmp9_)), self->priv->preferred_share = (g_free (self->priv->preferred_share), NULL), _tmp10_);
				}
				cols = (_vala_array_free (cols, cols_length1, (GDestroyNotify) g_free), NULL);
				local_file = (g_free (local_file), NULL);
				shared_as = (g_free (shared_as), NULL);
			}
		}
	}
	rows = (_vala_array_free (rows, rows_length1, (GDestroyNotify) g_free), NULL);
}


char* _dynamic_get_paths_as_string8 (DBusGProxy* self, GError** error) {
	char* result;
	dbus_g_proxy_call (self, "GetPathsAsString", error, G_TYPE_INVALID, G_TYPE_STRING, &result, G_TYPE_INVALID);
	if (*error) {
		return NULL;
	}
	return result;
}


gint _dynamic_get_redirection_type9 (DBusGProxy* self, GError** error) {
	gint result;
	dbus_g_proxy_call (self, "GetRedirectionType", error, G_TYPE_INVALID, G_TYPE_INT, &result, G_TYPE_INVALID);
	if (*error) {
		return 0;
	}
	return result;
}


gint _dynamic_get_redirection_status10 (DBusGProxy* self, GError** error) {
	gint result;
	dbus_g_proxy_call (self, "GetRedirectionStatus", error, G_TYPE_INVALID, G_TYPE_INT, &result, G_TYPE_INVALID);
	if (*error) {
		return 0;
	}
	return result;
}


static void gui_update_model (Gui* self) {
	GError * _inner_error_;
	gint int_redirection_status;
	gint int_redirection_type;
	gboolean enable_redirection_options;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	/* Setting REDIRECTION_STATUS_PENDING ensures that the options will
	 be shaded unless the server confirms its status*/
	int_redirection_status = 1;
	int_redirection_type = 0;
	enable_redirection_options = FALSE;
	_tmp0_ = NULL;
	self->priv->string_model = (_tmp0_ = NULL, self->priv->string_model = (g_free (self->priv->string_model), NULL), _tmp0_);
	if (gui_get_remote (self) != NULL) {
		{
			char* _tmp1_;
			char* _tmp2_;
			_tmp1_ = _dynamic_get_paths_as_string8 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch8_g_error;
				goto __finally8;
			}
			_tmp2_ = NULL;
			self->priv->string_model = (_tmp2_ = _tmp1_, self->priv->string_model = (g_free (self->priv->string_model), NULL), _tmp2_);
		}
		goto __finally8;
		__catch8_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				gui_log (self, _ ("Remote error getting paths\n"));
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally8:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
		{
			gint _tmp3_;
			gint _tmp4_;
			_tmp3_ = _dynamic_get_redirection_type9 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch9_g_error;
				goto __finally9;
			}
			int_redirection_type = _tmp3_;
			_tmp4_ = _dynamic_get_redirection_status10 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch9_g_error;
				goto __finally9;
			}
			int_redirection_status = _tmp4_;
		}
		goto __finally9;
		__catch9_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				gui_log (self, _ ("Remote error getting redirection options\n"));
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally9:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	if (self->priv->string_model == NULL) {
		char* _tmp5_;
		_tmp5_ = NULL;
		self->priv->string_model = (_tmp5_ = g_strdup (""), self->priv->string_model = (g_free (self->priv->string_model), NULL), _tmp5_);
	}
	gui_update_model_from_string (self, self->priv->model, self->priv->string_model);
	gtk_combo_box_set_active (self->priv->redirection_type, int_redirection_type);
	/* Disable redirection settings when redirection process is pending*/
	enable_redirection_options = int_redirection_status != 1;
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->redirection_type, enable_redirection_options);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->ssh_host, enable_redirection_options);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->ssh_port, enable_redirection_options);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->ssh_user, enable_redirection_options);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->ssh_password, enable_redirection_options);
	gtk_widget_set_sensitive ((GtkWidget*) self->priv->redirection_apply, enable_redirection_options);
}


char* _dynamic_get_pending_log11 (DBusGProxy* self, guint param1, GError** error) {
	char* result;
	dbus_g_proxy_call (self, "GetPendingLog", error, G_TYPE_UINT, param1, G_TYPE_INVALID, G_TYPE_STRING, &result, G_TYPE_INVALID);
	if (*error) {
		return NULL;
	}
	return result;
}


static void gui_update_log (Gui* self) {
	GError * _inner_error_;
	char* new_log_lines;
	guint j;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	new_log_lines = NULL;
	if (gui_get_remote (self) != NULL) {
		{
			char* _tmp0_;
			char* _tmp1_;
			_tmp0_ = _dynamic_get_pending_log11 (gui_get_remote (self), self->priv->lastlog, &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch10_g_error;
				goto __finally10;
			}
			_tmp1_ = NULL;
			new_log_lines = (_tmp1_ = _tmp0_, new_log_lines = (g_free (new_log_lines), NULL), _tmp1_);
		}
		goto __finally10;
		__catch10_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				gui_log (self, _ ("Remote error getting log lines\n"));
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally10:
		if (_inner_error_ != NULL) {
			new_log_lines = (g_free (new_log_lines), NULL);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	if (new_log_lines == NULL) {
		char* _tmp2_;
		_tmp2_ = NULL;
		new_log_lines = (_tmp2_ = g_strdup (""), new_log_lines = (g_free (new_log_lines), NULL), _tmp2_);
	}
	/* Count the number of new lines*/
	j = (guint) 0;
	{
		guint i;
		i = (guint) 0;
		{
			gboolean _tmp3_;
			_tmp3_ = TRUE;
			while (TRUE) {
				if (!_tmp3_) {
					i++;
				}
				_tmp3_ = FALSE;
				if (!(g_utf8_get_char (g_utf8_offset_to_pointer (new_log_lines, i)) != '\0')) {
					break;
				}
				if (g_utf8_get_char (g_utf8_offset_to_pointer (new_log_lines, i)) == '\n') {
					j++;
				}
			}
		}
	}
	gui_log (self, new_log_lines);
	self->priv->lastlog = self->priv->lastlog + j;
	new_log_lines = (g_free (new_log_lines), NULL);
}


void gui_on_remote_log_changed (Gui* self) {
	g_return_if_fail (self != NULL);
	gui_update_log (self);
}


gint _dynamic_get_redirection_status12 (DBusGProxy* self, GError** error) {
	gint result;
	dbus_g_proxy_call (self, "GetRedirectionStatus", error, G_TYPE_INVALID, G_TYPE_INT, &result, G_TYPE_INVALID);
	if (*error) {
		return 0;
	}
	return result;
}


char* _dynamic_get_public_url13 (DBusGProxy* self, GError** error) {
	char* result;
	dbus_g_proxy_call (self, "GetPublicUrl", error, G_TYPE_INVALID, G_TYPE_STRING, &result, G_TYPE_INVALID);
	if (*error) {
		return NULL;
	}
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


char* _dynamic_get_requests_stats14 (DBusGProxy* self, GError** error) {
	char* result;
	dbus_g_proxy_call (self, "GetRequestsStats", error, G_TYPE_INVALID, G_TYPE_STRING, &result, G_TYPE_INVALID);
	if (*error) {
		return NULL;
	}
	return result;
}


static void gui_update_statusbar (Gui* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	if (gui_get_remote (self) != NULL) {
		{
			gint status;
			char* string_status;
			char* _tmp0_;
			char* _tmp1_;
			gboolean _tmp2_;
			gboolean _tmp3_;
			char* _tmp8_;
			char* _tmp10_;
			char* _tmp9_;
			char* _tmp11_;
			char* _tmp16_;
			status = _dynamic_get_redirection_status12 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch11_g_error;
				goto __finally11;
			}
			string_status = NULL;
			_tmp0_ = _dynamic_get_public_url13 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				string_status = (g_free (string_status), NULL);
				goto __catch11_g_error;
				goto __finally11;
			}
			_tmp1_ = NULL;
			self->priv->public_url = (_tmp1_ = _tmp0_, self->priv->public_url = (g_free (self->priv->public_url), NULL), _tmp1_);
			_tmp2_ = FALSE;
			_tmp3_ = FALSE;
			if (self->priv->public_url != NULL) {
				_tmp3_ = string_get_length (self->priv->public_url) > 0;
			} else {
				_tmp3_ = FALSE;
			}
			if (_tmp3_) {
				_tmp2_ = self->priv->preferred_share != NULL;
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				char* _tmp4_;
				char* _tmp5_;
				_tmp4_ = NULL;
				self->priv->invitation = (_tmp4_ = g_strconcat (self->priv->public_url, self->priv->preferred_share, NULL), self->priv->invitation = (g_free (self->priv->invitation), NULL), _tmp4_);
				_tmp5_ = NULL;
				string_status = (_tmp5_ = g_strdup (" - "), string_status = (g_free (string_status), NULL), _tmp5_);
			} else {
				char* _tmp6_;
				char* _tmp7_;
				_tmp6_ = NULL;
				self->priv->invitation = (_tmp6_ = g_strdup (""), self->priv->invitation = (g_free (self->priv->invitation), NULL), _tmp6_);
				_tmp7_ = NULL;
				string_status = (_tmp7_ = g_strdup (""), string_status = (g_free (string_status), NULL), _tmp7_);
			}
			_tmp8_ = _dynamic_get_requests_stats14 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				string_status = (g_free (string_status), NULL);
				goto __catch11_g_error;
				goto __finally11;
			}
			_tmp10_ = NULL;
			_tmp9_ = NULL;
			string_status = (_tmp10_ = g_strconcat (string_status, _tmp9_ = _tmp8_, NULL), string_status = (g_free (string_status), NULL), _tmp10_);
			_tmp9_ = (g_free (_tmp9_), NULL);
			_tmp11_ = NULL;
			string_status = (_tmp11_ = g_strconcat (string_status, " - ", NULL), string_status = (g_free (string_status), NULL), _tmp11_);
			switch (status) {
				case 0:
				{
					char* _tmp12_;
					_tmp12_ = NULL;
					string_status = (_tmp12_ = g_strconcat (string_status, _ ("Direct connection"), NULL), string_status = (g_free (string_status), NULL), _tmp12_);
					break;
				}
				case 1:
				{
					char* _tmp13_;
					_tmp13_ = NULL;
					string_status = (_tmp13_ = g_strconcat (string_status, _ ("Performing redirection"), NULL), string_status = (g_free (string_status), NULL), _tmp13_);
					break;
				}
				case 2:
				{
					char* _tmp14_;
					_tmp14_ = NULL;
					string_status = (_tmp14_ = g_strconcat (string_status, _ ("Redirection performed"), NULL), string_status = (g_free (string_status), NULL), _tmp14_);
					break;
				}
				case 3:
				{
					char* _tmp15_;
					_tmp15_ = NULL;
					string_status = (_tmp15_ = g_strconcat (string_status, _ ("Redirection error"), NULL), string_status = (g_free (string_status), NULL), _tmp15_);
					break;
				}
			}
			gtk_statusbar_pop (self->priv->statusbar, (guint) 0);
			_tmp16_ = NULL;
			gtk_statusbar_push (self->priv->statusbar, (guint) 0, _tmp16_ = g_strconcat (self->priv->invitation, string_status, NULL));
			_tmp16_ = (g_free (_tmp16_), NULL);
			string_status = (g_free (string_status), NULL);
		}
		goto __finally11;
		__catch11_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				gui_log (self, _ ("Remote error getting public url\n"));
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally11:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


static void _g_list_free_gtk_tree_path_free (GList* self) {
	g_list_foreach (self, (GFunc) gtk_tree_path_free, NULL);
	g_list_free (self);
}


static GList* gui_get_selected_shares (Gui* self) {
	GList* result;
	GList* selection;
	g_return_val_if_fail (self != NULL, NULL);
	selection = NULL;
	{
		GList* spath_collection;
		GList* spath_it;
		spath_collection = gtk_tree_selection_get_selected_rows (gtk_tree_view_get_selection (self->priv->files), NULL);
		for (spath_it = spath_collection; spath_it != NULL; spath_it = spath_it->next) {
			const GtkTreePath* spath;
			spath = (const GtkTreePath*) spath_it->data;
			{
				GtkTreeIter iter = {0};
				if (gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->model, &iter, spath)) {
					char* shared_as;
					shared_as = NULL;
					gtk_tree_model_get ((GtkTreeModel*) self->priv->model, &iter, 1, &shared_as, -1, -1);
					if (shared_as != NULL) {
						const char* _tmp0_;
						_tmp0_ = NULL;
						selection = g_list_append (selection, (_tmp0_ = shared_as, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)));
					}
					shared_as = (g_free (shared_as), NULL);
				}
			}
		}
		(spath_collection == NULL) ? NULL : (spath_collection = (_g_list_free_gtk_tree_path_free (spath_collection), NULL));
	}
	result = selection;
	return result;
}


/* Public methods*/
void gui_init (Gui* self) {
	g_return_if_fail (self != NULL);
	gui_gui_init (self);
	gui_nautilus_init (self);
}


void _dynamic_shutdown15 (DBusGProxy* self, GError** error) {
	dbus_g_proxy_call (self, "Shutdown", error, G_TYPE_INVALID, G_TYPE_INVALID);
	if (*error) {
		return;
	}
}


void gui_quit (Gui* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	gtk_main_quit ();
	if (gui_get_remote (self) != NULL) {
		{
			_dynamic_shutdown15 (gui_get_remote (self), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch12_g_error;
				goto __finally12;
			}
		}
		goto __finally12;
		__catch12_g_error:
		{
			GError * e;
			e = _inner_error_;
			_inner_error_ = NULL;
			{
				gui_log (self, _ ("Remote error shutting down server\n"));
				(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
			}
		}
		__finally12:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
}


gint gui_main (char** args, int args_length1) {
	gint result;
	Gui* gui;
	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
	gui = gui_new ();
	gtk_init (&args_length1, &args);
	gui_init (gui);
	gtk_main ();
	result = 0;
	(gui == NULL) ? NULL : (gui = (g_object_unref (gui), NULL));
	return result;
}


int main (int argc, char ** argv) {
	g_thread_init (NULL);
	g_type_init ();
	return gui_main (argv, argc);
}


Gui* gui_construct (GType object_type) {
	Gui * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


Gui* gui_new (void) {
	return gui_construct (TYPE_GUI);
}


static DBusGProxy* gui_get_remote (Gui* self) {
	DBusGProxy* result;
	g_return_val_if_fail (self != NULL, NULL);
	/* Maybe remote DBUS service isn't immediately available, so by using
	 this lazy init we give it the opportunity to be contacted each time
	 we call for it*/
	if (self->priv->_remote == NULL) {
		gui_dbus_init (self);
	}
	result = self->priv->_remote;
	return result;
}


static void gui_class_init (GuiClass * klass) {
	gui_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GuiPrivate));
	G_OBJECT_CLASS (klass)->get_property = gui_get_property;
	G_OBJECT_CLASS (klass)->finalize = gui_finalize;
}


static void gui_instance_init (Gui * self) {
	self->priv = GUI_GET_PRIVATE (self);
	self->priv->_remote = NULL;
}


static void gui_finalize (GObject* obj) {
	Gui * self;
	self = GUI (obj);
	(self->priv->builder == NULL) ? NULL : (self->priv->builder = (g_object_unref (self->priv->builder), NULL));
	(self->priv->menu == NULL) ? NULL : (self->priv->menu = (g_object_unref (self->priv->menu), NULL));
	(self->priv->systraymenu == NULL) ? NULL : (self->priv->systraymenu = (g_object_unref (self->priv->systraymenu), NULL));
	(self->priv->top == NULL) ? NULL : (self->priv->top = (g_object_unref (self->priv->top), NULL));
	(self->priv->adddialog == NULL) ? NULL : (self->priv->adddialog = (g_object_unref (self->priv->adddialog), NULL));
	(self->priv->aboutdialog == NULL) ? NULL : (self->priv->aboutdialog = (g_object_unref (self->priv->aboutdialog), NULL));
	(self->priv->systray == NULL) ? NULL : (self->priv->systray = (g_object_unref (self->priv->systray), NULL));
	(self->priv->systraymenu_restore == NULL) ? NULL : (self->priv->systraymenu_restore = (g_object_unref (self->priv->systraymenu_restore), NULL));
	(self->priv->files == NULL) ? NULL : (self->priv->files = (g_object_unref (self->priv->files), NULL));
	(self->priv->redirection_type == NULL) ? NULL : (self->priv->redirection_type = (g_object_unref (self->priv->redirection_type), NULL));
	(self->priv->ssh_host == NULL) ? NULL : (self->priv->ssh_host = (g_object_unref (self->priv->ssh_host), NULL));
	(self->priv->ssh_port == NULL) ? NULL : (self->priv->ssh_port = (g_object_unref (self->priv->ssh_port), NULL));
	(self->priv->ssh_user == NULL) ? NULL : (self->priv->ssh_user = (g_object_unref (self->priv->ssh_user), NULL));
	(self->priv->ssh_password == NULL) ? NULL : (self->priv->ssh_password = (g_object_unref (self->priv->ssh_password), NULL));
	(self->priv->redirection_apply == NULL) ? NULL : (self->priv->redirection_apply = (g_object_unref (self->priv->redirection_apply), NULL));
	(self->priv->statusbar == NULL) ? NULL : (self->priv->statusbar = (g_object_unref (self->priv->statusbar), NULL));
	(self->priv->localdirectory == NULL) ? NULL : (self->priv->localdirectory = (g_object_unref (self->priv->localdirectory), NULL));
	(self->priv->shareas == NULL) ? NULL : (self->priv->shareas = (g_object_unref (self->priv->shareas), NULL));
	(self->priv->logtext == NULL) ? NULL : (self->priv->logtext = (g_object_unref (self->priv->logtext), NULL));
	(self->priv->clipboard == NULL) ? NULL : (self->priv->clipboard = (g_object_unref (self->priv->clipboard), NULL));
	(self->priv->model == NULL) ? NULL : (self->priv->model = (g_object_unref (self->priv->model), NULL));
	self->priv->string_model = (g_free (self->priv->string_model), NULL);
	self->priv->public_url = (g_free (self->priv->public_url), NULL);
	self->priv->preferred_share = (g_free (self->priv->preferred_share), NULL);
	self->priv->invitation = (g_free (self->priv->invitation), NULL);
	self->priv->display = (g_free (self->priv->display), NULL);
	(self->priv->_remote == NULL) ? NULL : (self->priv->_remote = (g_object_unref (self->priv->_remote), NULL));
	G_OBJECT_CLASS (gui_parent_class)->finalize (obj);
}


GType gui_get_type (void) {
	static GType gui_type_id = 0;
	if (gui_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GuiClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gui_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Gui), 0, (GInstanceInitFunc) gui_instance_init, NULL };
		gui_type_id = g_type_register_static (G_TYPE_OBJECT, "Gui", &g_define_type_info, 0);
	}
	return gui_type_id;
}


static void gui_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	Gui * self;
	gpointer boxed;
	self = GUI (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




