/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>


#define TYPE_MEIGA (meiga_get_type ())
#define MEIGA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEIGA, Meiga))
#define MEIGA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEIGA, MeigaClass))
#define IS_MEIGA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEIGA))
#define IS_MEIGA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEIGA))
#define MEIGA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEIGA, MeigaClass))

typedef struct _Meiga Meiga;
typedef struct _MeigaClass MeigaClass;
typedef struct _MeigaPrivate MeigaPrivate;

#define TYPE_MEIGA_SERVER (meiga_server_get_type ())
#define MEIGA_SERVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEIGA_SERVER, MeigaServer))
#define MEIGA_SERVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEIGA_SERVER, MeigaServerClass))
#define IS_MEIGA_SERVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEIGA_SERVER))
#define IS_MEIGA_SERVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEIGA_SERVER))
#define MEIGA_SERVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEIGA_SERVER, MeigaServerClass))

typedef struct _MeigaServer MeigaServer;
typedef struct _MeigaServerClass MeigaServerClass;

#define TYPE_LOG (log_get_type ())
#define LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LOG, Log))
#define LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LOG, LogClass))
#define IS_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LOG))
#define IS_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LOG))
#define LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LOG, LogClass))

typedef struct _Log Log;
typedef struct _LogClass LogClass;
typedef struct _DBusObjectVTable _DBusObjectVTable;

struct _Meiga {
	GObject parent_instance;
	MeigaPrivate * priv;
};

struct _MeigaClass {
	GObjectClass parent_class;
};

struct _MeigaPrivate {
	MeigaServer* server;
};

struct _DBusObjectVTable {
	void (*register_object) (DBusConnection*, const char*, void*);
};


static gpointer meiga_parent_class = NULL;

GType meiga_get_type (void);
GType meiga_server_get_type (void);
#define MEIGA_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MEIGA, MeigaPrivate))
enum  {
	MEIGA_DUMMY_PROPERTY
};
GType log_get_type (void);
Log* meiga_server_get_logger (MeigaServer* self);
static void _lambda0_ (Log* o, Meiga* self);
static void __lambda0__log_changed (Log* _sender, gpointer self);
static void _lambda1_ (MeigaServer* o, Meiga* self);
static void __lambda1__meiga_server_model_changed (MeigaServer* _sender, gpointer self);
Meiga* meiga_new (MeigaServer* server);
Meiga* meiga_construct (GType object_type, MeigaServer* server);
void meiga_server_register_path (MeigaServer* self, const char* real_path, const char* logical_path);
void meiga_register_path (Meiga* self, const char* real_path, const char* logical_path);
void meiga_server_unregister_path (MeigaServer* self, const char* logical_path);
void meiga_unregister_path (Meiga* self, const char* logical_path);
guint meiga_server_get_gui_pid (MeigaServer* self);
guint meiga_get_gui_pid (Meiga* self);
const char* meiga_server_get_display (MeigaServer* self);
char* meiga_get_display (Meiga* self);
void meiga_server_register_gui (MeigaServer* self, guint gui_pid, const char* display);
void meiga_register_gui (Meiga* self, guint gui_pid, const char* display);
GHashTable* meiga_server_get_paths (MeigaServer* self);
GHashTable* meiga_get_paths (Meiga* self);
char* meiga_get_paths_as_string (Meiga* self);
char* meiga_server_get_public_url (MeigaServer* self);
char* meiga_get_public_url (Meiga* self);
gint meiga_server_get_redirection_type (MeigaServer* self);
gint meiga_get_redirection_type (Meiga* self);
void meiga_server_set_redirection_type (MeigaServer* self, gint redirection_type);
void meiga_set_redirection_type (Meiga* self, gint redirection_type);
gint meiga_server_get_redirection_status (MeigaServer* self);
gint meiga_get_redirection_status (Meiga* self);
void meiga_server_set_ssh_host (MeigaServer* self, const char* ssh_host);
void meiga_set_ssh_host (Meiga* self, const char* ssh_host);
void meiga_server_set_ssh_port (MeigaServer* self, const char* ssh_port);
void meiga_set_ssh_port (Meiga* self, const char* ssh_port);
void meiga_server_set_ssh_user (MeigaServer* self, const char* ssh_user);
void meiga_set_ssh_user (Meiga* self, const char* ssh_user);
void meiga_server_set_ssh_password (MeigaServer* self, const char* ssh_password);
void meiga_set_ssh_password (Meiga* self, const char* ssh_password);
void meiga_server_shutdown (MeigaServer* self);
void meiga_shutdown (Meiga* self);
char* log_get_pending (Log* self, guint start);
char* meiga_get_pending_log (Meiga* self, guint start);
char* meiga_server_get_requests_stats (MeigaServer* self);
char* meiga_get_requests_stats (Meiga* self);
void meiga_dbus_register_object (DBusConnection* connection, const char* path, void* object);
void _meiga_dbus_unregister (DBusConnection* connection, void* user_data);
DBusHandlerResult meiga_dbus_message (DBusConnection* connection, DBusMessage* message, void* object);
static DBusMessage* _dbus_meiga_introspect (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_property_get_all (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_register_path (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_unregister_path (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_gui_pid (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_display (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_register_gui (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_paths (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_paths_as_string (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_public_url (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_redirection_type (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_set_redirection_type (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_redirection_status (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_set_ssh_host (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_set_ssh_port (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_set_ssh_user (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_set_ssh_password (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_shutdown (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_pending_log (Meiga* self, DBusConnection* connection, DBusMessage* message);
static DBusMessage* _dbus_meiga_get_requests_stats (Meiga* self, DBusConnection* connection, DBusMessage* message);
static void _dbus_meiga_model_changed (GObject* _sender, DBusConnection* _connection);
static void _dbus_meiga_log_changed (GObject* _sender, DBusConnection* _connection);
static void meiga_finalize (GObject* obj);
static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object);
static void _vala_dbus_unregister_object (gpointer connection, GObject* object);

static const DBusObjectPathVTable _meiga_dbus_path_vtable = {_meiga_dbus_unregister, meiga_dbus_message};
static const _DBusObjectVTable _meiga_dbus_vtable = {meiga_dbus_register_object};


static void _lambda0_ (Log* o, Meiga* self) {
	g_return_if_fail (o != NULL);
	g_signal_emit_by_name (self, "log-changed");
}


static void __lambda0__log_changed (Log* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static void _lambda1_ (MeigaServer* o, Meiga* self) {
	g_return_if_fail (o != NULL);
	g_signal_emit_by_name (self, "model-changed");
}


static void __lambda1__meiga_server_model_changed (MeigaServer* _sender, gpointer self) {
	_lambda1_ (_sender, self);
}


Meiga* meiga_construct (GType object_type, MeigaServer* server) {
	Meiga * self;
	MeigaServer* _tmp1_;
	MeigaServer* _tmp0_;
	g_return_val_if_fail (server != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->server = (_tmp1_ = (_tmp0_ = server, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->server == NULL) ? NULL : (self->priv->server = (g_object_unref (self->priv->server), NULL)), _tmp1_);
	if (meiga_server_get_logger (self->priv->server) != NULL) {
		g_signal_connect (meiga_server_get_logger (self->priv->server), "changed", (GCallback) __lambda0__log_changed, self);
	}
	g_signal_connect (self->priv->server, "model-changed", (GCallback) __lambda1__meiga_server_model_changed, self);
	return self;
}


Meiga* meiga_new (MeigaServer* server) {
	return meiga_construct (TYPE_MEIGA, server);
}


void meiga_register_path (Meiga* self, const char* real_path, const char* logical_path) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (real_path != NULL);
	g_return_if_fail (logical_path != NULL);
	meiga_server_register_path (self->priv->server, real_path, logical_path);
	g_signal_emit_by_name (self, "model-changed");
}


void meiga_unregister_path (Meiga* self, const char* logical_path) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (logical_path != NULL);
	meiga_server_unregister_path (self->priv->server, logical_path);
	g_signal_emit_by_name (self, "model-changed");
}


guint meiga_get_gui_pid (Meiga* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = meiga_server_get_gui_pid (self->priv->server);
	return result;
}


char* meiga_get_display (Meiga* self) {
	char* result;
	const char* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = meiga_server_get_display (self->priv->server), (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_));
	return result;
}


void meiga_register_gui (Meiga* self, guint gui_pid, const char* display) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (display != NULL);
	meiga_server_register_gui (self->priv->server, gui_pid, display);
}


GHashTable* meiga_get_paths (Meiga* self) {
	GHashTable* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = meiga_server_get_paths (self->priv->server);
	return result;
}


char* meiga_get_paths_as_string (Meiga* self) {
	char* result;
	GHashTable* t;
	GList* _tmp0_;
	GList* _tmp1_;
	GList* keys;
	char* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	t = meiga_server_get_paths (self->priv->server);
	_tmp0_ = NULL;
	_tmp1_ = NULL;
	keys = (_tmp1_ = g_list_copy (_tmp0_ = g_hash_table_get_keys (t)), (_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_list_free (_tmp0_), NULL)), _tmp1_);
	keys = g_list_sort (keys, strcmp);
	_result_ = g_strdup ("");
	{
		GList* k_collection;
		GList* k_it;
		k_collection = keys;
		for (k_it = k_collection; k_it != NULL; k_it = k_it->next) {
			const char* k;
			k = (const char*) k_it->data;
			{
				const char* _tmp2_;
				char* v;
				char* _tmp6_;
				char* _tmp5_;
				char* _tmp4_;
				char* _tmp3_;
				_tmp2_ = NULL;
				v = (_tmp2_ = (const char*) g_hash_table_lookup (t, k), (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
				_tmp6_ = NULL;
				_tmp5_ = NULL;
				_tmp4_ = NULL;
				_tmp3_ = NULL;
				_result_ = (_tmp6_ = g_strconcat (_result_, _tmp5_ = g_strconcat (_tmp4_ = g_strconcat (_tmp3_ = g_strconcat (k, "\t", NULL), v, NULL), "\n", NULL), NULL), _result_ = (g_free (_result_), NULL), _tmp6_);
				_tmp5_ = (g_free (_tmp5_), NULL);
				_tmp4_ = (g_free (_tmp4_), NULL);
				_tmp3_ = (g_free (_tmp3_), NULL);
				v = (g_free (v), NULL);
			}
		}
	}
	result = _result_;
	(t == NULL) ? NULL : (t = (g_hash_table_unref (t), NULL));
	(keys == NULL) ? NULL : (keys = (g_list_free (keys), NULL));
	return result;
}


char* meiga_get_public_url (Meiga* self) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = meiga_server_get_public_url (self->priv->server);
	return result;
}


gint meiga_get_redirection_type (Meiga* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = meiga_server_get_redirection_type (self->priv->server);
	return result;
}


void meiga_set_redirection_type (Meiga* self, gint redirection_type) {
	g_return_if_fail (self != NULL);
	meiga_server_set_redirection_type (self->priv->server, redirection_type);
}


gint meiga_get_redirection_status (Meiga* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = meiga_server_get_redirection_status (self->priv->server);
	return result;
}


void meiga_set_ssh_host (Meiga* self, const char* ssh_host) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_host != NULL);
	meiga_server_set_ssh_host (self->priv->server, ssh_host);
}


void meiga_set_ssh_port (Meiga* self, const char* ssh_port) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_port != NULL);
	meiga_server_set_ssh_port (self->priv->server, ssh_port);
}


void meiga_set_ssh_user (Meiga* self, const char* ssh_user) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_user != NULL);
	meiga_server_set_ssh_user (self->priv->server, ssh_user);
}


void meiga_set_ssh_password (Meiga* self, const char* ssh_password) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_password != NULL);
	meiga_server_set_ssh_password (self->priv->server, ssh_password);
}


void meiga_shutdown (Meiga* self) {
	g_return_if_fail (self != NULL);
	meiga_server_shutdown (self->priv->server);
}


char* meiga_get_pending_log (Meiga* self, guint start) {
	char* result;
	char* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	_result_ = NULL;
	if (meiga_server_get_logger (self->priv->server) != NULL) {
		char* _tmp0_;
		_tmp0_ = NULL;
		_result_ = (_tmp0_ = log_get_pending (meiga_server_get_logger (self->priv->server), start), _result_ = (g_free (_result_), NULL), _tmp0_);
	} else {
		if (start == 0) {
			char* _tmp2_;
			const char* _tmp1_;
			_tmp2_ = NULL;
			_tmp1_ = NULL;
			_result_ = (_tmp2_ = (_tmp1_ = _ ("--- No log available ---"), (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)), _result_ = (g_free (_result_), NULL), _tmp2_);
		} else {
			char* _tmp3_;
			_tmp3_ = NULL;
			_result_ = (_tmp3_ = g_strdup (""), _result_ = (g_free (_result_), NULL), _tmp3_);
		}
	}
	result = _result_;
	return result;
}


char* meiga_get_requests_stats (Meiga* self) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = meiga_server_get_requests_stats (self->priv->server);
	return result;
}


void _meiga_dbus_unregister (DBusConnection* connection, void* user_data) {
}


static DBusMessage* _dbus_meiga_introspect (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessage* reply;
	DBusMessageIter iter;
	GString* xml_data;
	char** children;
	int i;
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	xml_data = g_string_new ("<!DOCTYPE node PUBLIC \"-//freedesktop//DTD D-BUS Object Introspection 1.0//EN\" \"http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd\">\n");
	g_string_append (xml_data, "<node>\n<interface name=\"org.freedesktop.DBus.Introspectable\">\n  <method name=\"Introspect\">\n    <arg name=\"data\" direction=\"out\" type=\"s\"/>\n  </method>\n</interface>\n<interface name=\"org.freedesktop.DBus.Properties\">\n  <method name=\"Get\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"out\" type=\"v\"/>\n  </method>\n  <method name=\"Set\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"propname\" direction=\"in\" type=\"s\"/>\n    <arg name=\"value\" direction=\"in\" type=\"v\"/>\n  </method>\n  <method name=\"GetAll\">\n    <arg name=\"interface\" direction=\"in\" type=\"s\"/>\n    <arg name=\"props\" direction=\"out\" type=\"a{sv}\"/>\n  </method>\n</interface>\n<interface name=\"com.igalia.Meiga\">\n  <method name=\"RegisterPath\">\n    <arg name=\"real_path\" type=\"s\" direction=\"in\"/>\n    <arg name=\"logical_path\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"UnregisterPath\">\n    <arg name=\"logical_path\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"GetGuiPid\">\n    <arg name=\"result\" type=\"u\" direction=\"out\"/>\n  </method>\n  <method name=\"GetDisplay\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"RegisterGui\">\n    <arg name=\"gui_pid\" type=\"u\" direction=\"in\"/>\n    <arg name=\"display\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"GetPaths\">\n    <arg name=\"result\" type=\"a{ss}\" direction=\"out\"/>\n  </method>\n  <method name=\"GetPathsAsString\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetPublicUrl\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetRedirectionType\">\n    <arg name=\"result\" type=\"i\" direction=\"out\"/>\n  </method>\n  <method name=\"SetRedirectionType\">\n    <arg name=\"redirection_type\" type=\"i\" direction=\"in\"/>\n  </method>\n  <method name=\"GetRedirectionStatus\">\n    <arg name=\"result\" type=\"i\" direction=\"out\"/>\n  </method>\n  <method name=\"SetSshHost\">\n    <arg name=\"ssh_host\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"SetSshPort\">\n    <arg name=\"ssh_port\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"SetSshUser\">\n    <arg name=\"ssh_user\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"SetSshPassword\">\n    <arg name=\"ssh_password\" type=\"s\" direction=\"in\"/>\n  </method>\n  <method name=\"Shutdown\">\n  </method>\n  <method name=\"GetPendingLog\">\n    <arg name=\"start\" type=\"u\" direction=\"in\"/>\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <method name=\"GetRequestsStats\">\n    <arg name=\"result\" type=\"s\" direction=\"out\"/>\n  </method>\n  <signal name=\"ModelChanged\">\n  </signal>\n  <signal name=\"LogChanged\">\n  </signal>\n</interface>\n");
	dbus_connection_list_registered (connection, g_object_get_data ((GObject *) self, "dbus_object_path"), &children);
	for (i = 0; children[i]; i++) {
		g_string_append_printf (xml_data, "<node name=\"%s\"/>\n", children[i]);
	}
	dbus_free_string_array (children);
	g_string_append (xml_data, "</node>\n");
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &xml_data->str);
	g_string_free (xml_data, TRUE);
	return reply;
}


static DBusMessage* _dbus_meiga_property_get_all (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessage* reply;
	DBusMessageIter iter, reply_iter, subiter;
	char* interface_name;
	const char* _tmp0_;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &reply_iter);
	dbus_message_iter_get_basic (&iter, &_tmp0_);
	dbus_message_iter_next (&iter);
	interface_name = g_strdup (_tmp0_);
	if (strcmp (interface_name, "com.igalia.Meiga") == 0) {
		dbus_message_iter_open_container (&reply_iter, DBUS_TYPE_ARRAY, "{sv}", &subiter);
		dbus_message_iter_close_container (&reply_iter, &subiter);
	} else {
		dbus_message_unref (reply);
		reply = NULL;
	}
	g_free (interface_name);
	return reply;
}


static DBusMessage* _dbus_meiga_register_path (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* real_path;
	const char* _tmp1_;
	char* logical_path;
	const char* _tmp2_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "ss")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	real_path = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp1_);
	dbus_message_iter_next (&iter);
	real_path = g_strdup (_tmp1_);
	logical_path = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp2_);
	dbus_message_iter_next (&iter);
	logical_path = g_strdup (_tmp2_);
	meiga_register_path (self, real_path, logical_path);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	real_path = (g_free (real_path), NULL);
	logical_path = (g_free (logical_path), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_unregister_path (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* logical_path;
	const char* _tmp3_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	logical_path = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp3_);
	dbus_message_iter_next (&iter);
	logical_path = g_strdup (_tmp3_);
	meiga_unregister_path (self, logical_path);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	logical_path = (g_free (logical_path), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_get_gui_pid (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint result;
	DBusMessage* reply;
	dbus_uint32_t _tmp4_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_gui_pid (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp4_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &_tmp4_);
	return reply;
}


static DBusMessage* _dbus_meiga_get_display (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* result;
	DBusMessage* reply;
	const char* _tmp5_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_display (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp5_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp5_);
	result = (g_free (result), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_register_gui (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint gui_pid;
	dbus_uint32_t _tmp6_;
	char* display;
	const char* _tmp7_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "us")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	gui_pid = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp6_);
	dbus_message_iter_next (&iter);
	gui_pid = _tmp6_;
	display = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp7_);
	dbus_message_iter_next (&iter);
	display = g_strdup (_tmp7_);
	meiga_register_gui (self, gui_pid, display);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	display = (g_free (display), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_get_paths (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	GHashTable* result;
	DBusMessage* reply;
	DBusMessageIter _tmp8_, _tmp9_;
	GHashTableIter _tmp10_;
	gpointer _tmp11_, _tmp12_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_paths (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{ss}", &_tmp8_);
	g_hash_table_iter_init (&_tmp10_, result);
	while (g_hash_table_iter_next (&_tmp10_, &_tmp11_, &_tmp12_)) {
		char* _key;
		char* _value;
		const char* _tmp13_;
		const char* _tmp14_;
		dbus_message_iter_open_container (&_tmp8_, DBUS_TYPE_DICT_ENTRY, NULL, &_tmp9_);
		_key = (char*) _tmp11_;
		_value = (char*) _tmp12_;
		_tmp13_ = _key;
		dbus_message_iter_append_basic (&_tmp9_, DBUS_TYPE_STRING, &_tmp13_);
		_tmp14_ = _value;
		dbus_message_iter_append_basic (&_tmp9_, DBUS_TYPE_STRING, &_tmp14_);
		dbus_message_iter_close_container (&_tmp8_, &_tmp9_);
	}
	dbus_message_iter_close_container (&iter, &_tmp8_);
	(result == NULL) ? NULL : (result = (g_hash_table_unref (result), NULL));
	return reply;
}


static DBusMessage* _dbus_meiga_get_paths_as_string (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* result;
	DBusMessage* reply;
	const char* _tmp15_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_paths_as_string (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp15_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp15_);
	result = (g_free (result), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_get_public_url (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* result;
	DBusMessage* reply;
	const char* _tmp16_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_public_url (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp16_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp16_);
	result = (g_free (result), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_get_redirection_type (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	gint result;
	DBusMessage* reply;
	dbus_int32_t _tmp17_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_redirection_type (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp17_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &_tmp17_);
	return reply;
}


static DBusMessage* _dbus_meiga_set_redirection_type (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	gint redirection_type;
	dbus_int32_t _tmp18_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "i")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	redirection_type = 0;
	dbus_message_iter_get_basic (&iter, &_tmp18_);
	dbus_message_iter_next (&iter);
	redirection_type = _tmp18_;
	meiga_set_redirection_type (self, redirection_type);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	return reply;
}


static DBusMessage* _dbus_meiga_get_redirection_status (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	gint result;
	DBusMessage* reply;
	dbus_int32_t _tmp19_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_redirection_status (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp19_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &_tmp19_);
	return reply;
}


static DBusMessage* _dbus_meiga_set_ssh_host (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* ssh_host;
	const char* _tmp20_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	ssh_host = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp20_);
	dbus_message_iter_next (&iter);
	ssh_host = g_strdup (_tmp20_);
	meiga_set_ssh_host (self, ssh_host);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	ssh_host = (g_free (ssh_host), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_set_ssh_port (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* ssh_port;
	const char* _tmp21_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	ssh_port = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp21_);
	dbus_message_iter_next (&iter);
	ssh_port = g_strdup (_tmp21_);
	meiga_set_ssh_port (self, ssh_port);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	ssh_port = (g_free (ssh_port), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_set_ssh_user (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* ssh_user;
	const char* _tmp22_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	ssh_user = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp22_);
	dbus_message_iter_next (&iter);
	ssh_user = g_strdup (_tmp22_);
	meiga_set_ssh_user (self, ssh_user);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	ssh_user = (g_free (ssh_user), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_set_ssh_password (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* ssh_password;
	const char* _tmp23_;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "s")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	ssh_password = NULL;
	dbus_message_iter_get_basic (&iter, &_tmp23_);
	dbus_message_iter_next (&iter);
	ssh_password = g_strdup (_tmp23_);
	meiga_set_ssh_password (self, ssh_password);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	ssh_password = (g_free (ssh_password), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_shutdown (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	DBusMessage* reply;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	meiga_shutdown (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	return reply;
}


static DBusMessage* _dbus_meiga_get_pending_log (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	guint start;
	dbus_uint32_t _tmp24_;
	char* result;
	DBusMessage* reply;
	const char* _tmp25_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "u")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	start = 0U;
	dbus_message_iter_get_basic (&iter, &_tmp24_);
	dbus_message_iter_next (&iter);
	start = _tmp24_;
	result = meiga_get_pending_log (self, start);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp25_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp25_);
	result = (g_free (result), NULL);
	return reply;
}


static DBusMessage* _dbus_meiga_get_requests_stats (Meiga* self, DBusConnection* connection, DBusMessage* message) {
	DBusMessageIter iter;
	GError* error;
	char* result;
	DBusMessage* reply;
	const char* _tmp26_;
	error = NULL;
	if (strcmp (dbus_message_get_signature (message), "")) {
		return NULL;
	}
	dbus_message_iter_init (message, &iter);
	result = meiga_get_requests_stats (self);
	reply = dbus_message_new_method_return (message);
	dbus_message_iter_init_append (reply, &iter);
	_tmp26_ = result;
	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &_tmp26_);
	result = (g_free (result), NULL);
	return reply;
}


DBusHandlerResult meiga_dbus_message (DBusConnection* connection, DBusMessage* message, void* object) {
	DBusMessage* reply;
	reply = NULL;
	if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Introspectable", "Introspect")) {
		reply = _dbus_meiga_introspect (object, connection, message);
	} else if (dbus_message_is_method_call (message, "org.freedesktop.DBus.Properties", "GetAll")) {
		reply = _dbus_meiga_property_get_all (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "RegisterPath")) {
		reply = _dbus_meiga_register_path (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "UnregisterPath")) {
		reply = _dbus_meiga_unregister_path (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetGuiPid")) {
		reply = _dbus_meiga_get_gui_pid (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetDisplay")) {
		reply = _dbus_meiga_get_display (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "RegisterGui")) {
		reply = _dbus_meiga_register_gui (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetPaths")) {
		reply = _dbus_meiga_get_paths (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetPathsAsString")) {
		reply = _dbus_meiga_get_paths_as_string (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetPublicUrl")) {
		reply = _dbus_meiga_get_public_url (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetRedirectionType")) {
		reply = _dbus_meiga_get_redirection_type (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "SetRedirectionType")) {
		reply = _dbus_meiga_set_redirection_type (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetRedirectionStatus")) {
		reply = _dbus_meiga_get_redirection_status (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "SetSshHost")) {
		reply = _dbus_meiga_set_ssh_host (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "SetSshPort")) {
		reply = _dbus_meiga_set_ssh_port (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "SetSshUser")) {
		reply = _dbus_meiga_set_ssh_user (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "SetSshPassword")) {
		reply = _dbus_meiga_set_ssh_password (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "Shutdown")) {
		reply = _dbus_meiga_shutdown (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetPendingLog")) {
		reply = _dbus_meiga_get_pending_log (object, connection, message);
	} else if (dbus_message_is_method_call (message, "com.igalia.Meiga", "GetRequestsStats")) {
		reply = _dbus_meiga_get_requests_stats (object, connection, message);
	}
	if (reply) {
		dbus_connection_send (connection, reply, NULL);
		dbus_message_unref (reply);
		return DBUS_HANDLER_RESULT_HANDLED;
	} else {
		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
	}
}


static void _dbus_meiga_model_changed (GObject* _sender, DBusConnection* _connection) {
	const char * _path;
	DBusMessage *_message;
	DBusMessageIter _iter;
	_path = g_object_get_data (_sender, "dbus_object_path");
	_message = dbus_message_new_signal (_path, "com.igalia.Meiga", "ModelChanged");
	dbus_message_iter_init_append (_message, &_iter);
	dbus_connection_send (_connection, _message, NULL);
	dbus_message_unref (_message);
}


static void _dbus_meiga_log_changed (GObject* _sender, DBusConnection* _connection) {
	const char * _path;
	DBusMessage *_message;
	DBusMessageIter _iter;
	_path = g_object_get_data (_sender, "dbus_object_path");
	_message = dbus_message_new_signal (_path, "com.igalia.Meiga", "LogChanged");
	dbus_message_iter_init_append (_message, &_iter);
	dbus_connection_send (_connection, _message, NULL);
	dbus_message_unref (_message);
}


void meiga_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	if (!g_object_get_data (object, "dbus_object_path")) {
		g_object_set_data (object, "dbus_object_path", g_strdup (path));
		dbus_connection_register_object_path (connection, path, &_meiga_dbus_path_vtable, object);
		g_object_weak_ref (object, _vala_dbus_unregister_object, connection);
	}
	g_signal_connect (object, "model-changed", (GCallback) _dbus_meiga_model_changed, connection);
	g_signal_connect (object, "log-changed", (GCallback) _dbus_meiga_log_changed, connection);
}


static void meiga_class_init (MeigaClass * klass) {
	meiga_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (MeigaPrivate));
	G_OBJECT_CLASS (klass)->finalize = meiga_finalize;
	g_signal_new ("model_changed", TYPE_MEIGA, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("log_changed", TYPE_MEIGA, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_type_set_qdata (TYPE_MEIGA, g_quark_from_static_string ("DBusObjectVTable"), (void*) (&_meiga_dbus_vtable));
}


static void meiga_instance_init (Meiga * self) {
	self->priv = MEIGA_GET_PRIVATE (self);
}


static void meiga_finalize (GObject* obj) {
	Meiga * self;
	self = MEIGA (obj);
	(self->priv->server == NULL) ? NULL : (self->priv->server = (g_object_unref (self->priv->server), NULL));
	G_OBJECT_CLASS (meiga_parent_class)->finalize (obj);
}


GType meiga_get_type (void) {
	static GType meiga_type_id = 0;
	if (meiga_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (MeigaClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) meiga_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (Meiga), 0, (GInstanceInitFunc) meiga_instance_init, NULL };
		meiga_type_id = g_type_register_static (G_TYPE_OBJECT, "Meiga", &g_define_type_info, 0);
	}
	return meiga_type_id;
}


static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	const _DBusObjectVTable * vtable;
	vtable = g_type_get_qdata (G_TYPE_FROM_INSTANCE (object), g_quark_from_static_string ("DBusObjectVTable"));
	if (vtable) {
		vtable->register_object (connection, path, object);
	} else {
		g_warning ("Object does not implement any D-Bus interface");
	}
}


static void _vala_dbus_unregister_object (gpointer connection, GObject* object) {
	char* path;
	path = g_object_steal_data ((GObject*) object, "dbus_object_path");
	dbus_connection_unregister_object_path (connection, path);
	g_free (path);
}




