/* Meiga - Lightweight and easy to use web file server for your desktop
 *
 * Copyright (C) 2009 Igalia, S.L.
 *
 * Authors:
 *
 * Igalia, S.L. <info@igalia.com>
 * Enrique Ocaña González <eocanha@igalia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <libsoup/soup.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <locale.h>
#include <dbus/dbus.h>


#define TYPE_MEIGA_SERVER (meiga_server_get_type ())
#define MEIGA_SERVER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEIGA_SERVER, MeigaServer))
#define MEIGA_SERVER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEIGA_SERVER, MeigaServerClass))
#define IS_MEIGA_SERVER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEIGA_SERVER))
#define IS_MEIGA_SERVER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEIGA_SERVER))
#define MEIGA_SERVER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEIGA_SERVER, MeigaServerClass))

typedef struct _MeigaServer MeigaServer;
typedef struct _MeigaServerClass MeigaServerClass;
typedef struct _MeigaServerPrivate MeigaServerPrivate;

#define TYPE_MEIGA (meiga_get_type ())
#define MEIGA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_MEIGA, Meiga))
#define MEIGA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_MEIGA, MeigaClass))
#define IS_MEIGA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_MEIGA))
#define IS_MEIGA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_MEIGA))
#define MEIGA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_MEIGA, MeigaClass))

typedef struct _Meiga Meiga;
typedef struct _MeigaClass MeigaClass;

#define TYPE_NET (net_get_type ())
#define NET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_NET, Net))
#define NET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_NET, NetClass))
#define IS_NET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_NET))
#define IS_NET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_NET))
#define NET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_NET, NetClass))

typedef struct _Net Net;
typedef struct _NetClass NetClass;

#define TYPE_LOG (log_get_type ())
#define LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_LOG, Log))
#define LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_LOG, LogClass))
#define IS_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_LOG))
#define IS_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_LOG))
#define LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_LOG, LogClass))

typedef struct _Log Log;
typedef struct _LogClass LogClass;

#define TYPE_RSS_FEED (rss_feed_get_type ())
#define RSS_FEED(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_RSS_FEED, RssFeed))
#define RSS_FEED_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_RSS_FEED, RssFeedClass))
#define IS_RSS_FEED(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_RSS_FEED))
#define IS_RSS_FEED_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_RSS_FEED))
#define RSS_FEED_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), TYPE_RSS_FEED, RssFeedClass))

typedef struct _RssFeed RssFeed;
typedef struct _RssFeedClass RssFeedClass;
typedef struct _DBusObjectVTable _DBusObjectVTable;

struct _MeigaServer {
	GObject parent_instance;
	MeigaServerPrivate * priv;
};

struct _MeigaServerClass {
	GObjectClass parent_class;
};

struct _MeigaServerPrivate {
	gint port;
	SoupServer* server;
	GHashTable* path_mapping;
	GHashTable* mimetypes;
	Meiga* exposed;
	Net* net;
	guint pending_requests;
	guint total_requests;
	Log* _logger;
	guint _gui_pid;
	char* _display;
};

struct _DBusObjectVTable {
	void (*register_object) (DBusConnection*, const char*, void*);
};


static gpointer meiga_server_parent_class = NULL;

GType meiga_server_get_type (void);
GType meiga_get_type (void);
GType net_get_type (void);
GType log_get_type (void);
#define MEIGA_SERVER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TYPE_MEIGA_SERVER, MeigaServerPrivate))
enum  {
	MEIGA_SERVER_DUMMY_PROPERTY,
	MEIGA_SERVER_LOGGER,
	MEIGA_SERVER_GUI_PID,
	MEIGA_SERVER_DISPLAY
};
#define MEIGA_SERVER_MIME_TYPES_FILE "/etc/mime.types"
MeigaServer* meiga_server_new (void);
MeigaServer* meiga_server_construct (GType object_type);
Log* meiga_server_get_logger (MeigaServer* self);
void log_log (Log* self, const char* msg);
static void meiga_server_log (MeigaServer* self, const char* msg);
Log* log_new (void);
Log* log_construct (GType object_type);
static void meiga_server_set_logger (MeigaServer* self, Log* value);
static void meiga_server_initialize_mimetypes (MeigaServer* self);
void meiga_server_serve_file_callback_default (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client);
static void _meiga_server_serve_file_callback_default_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self);
void meiga_server_serve_rss_callback (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client);
static void _meiga_server_serve_rss_callback_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self);
static void meiga_server_initialize_dbus (MeigaServer* self);
void meiga_server_initialize (MeigaServer* self);
Net* net_new (void);
Net* net_construct (GType object_type);
void net_set_logger (Net* self, Log* value);
void net_set_port (Net* self, gint value);
const char* meiga_server_get_display (MeigaServer* self);
void net_set_display (Net* self, const char* value);
#define NET_REDIRECTION_TYPE_NONE 0
void net_set_redirection_type (Net* self, gint value);
char* meiga_server_get_public_url (MeigaServer* self);
static void _lambda9_ (Net* s, GParamSpec* p, MeigaServer* self);
static void __lambda9__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self);
static void _lambda10_ (Net* s, GParamSpec* p, MeigaServer* self);
static void __lambda10__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self);
static void _lambda11_ (Net* s, GParamSpec* p, MeigaServer* self);
static void __lambda11__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self);
void meiga_server_initialize_net (MeigaServer* self);
Meiga* meiga_new (MeigaServer* server);
Meiga* meiga_construct (GType object_type, MeigaServer* server);
guint _dynamic_request_name0 (DBusGProxy* self, const char* param1, guint param2, GError** error);
char* net_get_url (Net* self);
static gboolean _lambda12_ (MeigaServer* self);
static gboolean __lambda12__gsource_func (gpointer self);
void meiga_server_shutdown (MeigaServer* self);
void meiga_server_set_port (MeigaServer* self, gint port);
static void meiga_server_set_gui_pid (MeigaServer* self, guint value);
static void meiga_server_set_display (MeigaServer* self, const char* value);
void meiga_server_register_gui (MeigaServer* self, guint gui_pid, const char* display);
guint meiga_server_get_gui_pid (MeigaServer* self);
void meiga_server_serve_file_callback (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client);
static void _meiga_server_serve_file_callback_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self);
void meiga_server_register_path (MeigaServer* self, const char* real_path, const char* logical_path);
void meiga_server_unregister_path (MeigaServer* self, const char* logical_path);
gint net_get_redirection_type (Net* self);
gint meiga_server_get_redirection_type (MeigaServer* self);
void meiga_server_set_redirection_type (MeigaServer* self, gint redirection_type);
#define NET_REDIRECTION_STATUS_NONE 0
gint net_get_redirection_status (Net* self);
gint meiga_server_get_redirection_status (MeigaServer* self);
void net_set_ssh_host (Net* self, const char* value);
void meiga_server_set_ssh_host (MeigaServer* self, const char* ssh_host);
void net_set_ssh_port (Net* self, const char* value);
void meiga_server_set_ssh_port (MeigaServer* self, const char* ssh_port);
void net_set_ssh_user (Net* self, const char* value);
void meiga_server_set_ssh_user (MeigaServer* self, const char* ssh_user);
void net_set_ssh_password (Net* self, const char* value);
void meiga_server_set_ssh_password (MeigaServer* self, const char* ssh_password);
GHashTable* meiga_server_get_paths (MeigaServer* self);
char* meiga_server_get_requests_stats (MeigaServer* self);
static void _lambda2_ (MeigaServer* self);
static void __lambda2__soup_message_finished (SoupMessage* _sender, gpointer self);
static char* meiga_server_extension_from_path (MeigaServer* self, const char* path);
static void _g_list_free_g_free (GList* self);
GType rss_feed_get_type (void);
RssFeed* rss_feed_new_from_directory (const char* path, const char* base_url, const char* title, const char* description);
char* rss_feed_to_string (RssFeed* self);
void meiga_server_serve_error (MeigaServer* self, SoupMessage* msg, const char* path, SoupKnownStatusCode _error_, const char* error_message);
void meiga_server_main (char** args, int args_length1);
static void meiga_server_finalize (GObject* obj);
static void meiga_server_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void meiga_server_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);
static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object);
static void _vala_dbus_unregister_object (gpointer connection, GObject* object);



MeigaServer* meiga_server_construct (GType object_type) {
	MeigaServer * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


MeigaServer* meiga_server_new (void) {
	return meiga_server_construct (TYPE_MEIGA_SERVER);
}


static void meiga_server_log (MeigaServer* self, const char* msg) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	if (self->priv->_logger != NULL) {
		log_log (self->priv->_logger, msg);
	}
}


static void _meiga_server_serve_file_callback_default_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self) {
	meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
}


static void _meiga_server_serve_rss_callback_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self) {
	meiga_server_serve_rss_callback (self, server, msg, path, query, client);
}


void meiga_server_initialize (MeigaServer* self) {
	Log* _tmp0_;
	GHashTable* _tmp1_;
	SoupServer* _tmp2_;
	g_return_if_fail (self != NULL);
	_tmp0_ = NULL;
	meiga_server_set_logger (self, _tmp0_ = log_new ());
	(_tmp0_ == NULL) ? NULL : (_tmp0_ = (g_object_unref (_tmp0_), NULL));
	self->priv->port = 8001;
	self->priv->pending_requests = (guint) 0;
	self->priv->total_requests = (guint) 0;
	meiga_server_initialize_mimetypes (self);
	_tmp1_ = NULL;
	self->priv->path_mapping = (_tmp1_ = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->path_mapping == NULL) ? NULL : (self->priv->path_mapping = (g_hash_table_unref (self->priv->path_mapping), NULL)), _tmp1_);
	_tmp2_ = NULL;
	self->priv->server = (_tmp2_ = soup_server_new (SOUP_SERVER_PORT, self->priv->port, NULL), (self->priv->server == NULL) ? NULL : (self->priv->server = (g_object_unref (self->priv->server), NULL)), _tmp2_);
	soup_server_add_handler (self->priv->server, "/", _meiga_server_serve_file_callback_default_soup_server_callback, g_object_ref (self), g_object_unref);
	soup_server_add_handler (self->priv->server, "/rss", _meiga_server_serve_rss_callback_soup_server_callback, g_object_ref (self), g_object_unref);
	soup_server_run_async (self->priv->server);
	meiga_server_initialize_dbus (self);
}


static void _lambda9_ (Net* s, GParamSpec* p, MeigaServer* self) {
	char* _tmp1_;
	char* _tmp0_;
	g_return_if_fail (s != NULL);
	g_return_if_fail (p != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	meiga_server_log (self, _tmp1_ = g_strdup_printf (_ ("External URL: %s"), _tmp0_ = meiga_server_get_public_url (self)));
	_tmp1_ = (g_free (_tmp1_), NULL);
	_tmp0_ = (g_free (_tmp0_), NULL);
	g_signal_emit_by_name (self, "model-changed");
}


static void __lambda9__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self) {
	_lambda9_ (_sender, pspec, self);
}


static void _lambda10_ (Net* s, GParamSpec* p, MeigaServer* self) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (p != NULL);
	g_signal_emit_by_name (self, "model-changed");
}


static void __lambda10__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self) {
	_lambda10_ (_sender, pspec, self);
}


static void _lambda11_ (Net* s, GParamSpec* p, MeigaServer* self) {
	g_return_if_fail (s != NULL);
	g_return_if_fail (p != NULL);
	g_signal_emit_by_name (self, "model-changed");
}


static void __lambda11__g_object_notify (Net* _sender, GParamSpec* pspec, gpointer self) {
	_lambda11_ (_sender, pspec, self);
}


void meiga_server_initialize_net (MeigaServer* self) {
	Net* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp0_ = NULL;
	self->priv->net = (_tmp0_ = net_new (), (self->priv->net == NULL) ? NULL : (self->priv->net = (g_object_unref (self->priv->net), NULL)), _tmp0_);
	net_set_logger (self->priv->net, self->priv->_logger);
	net_set_port (self->priv->net, self->priv->port);
	net_set_display (self->priv->net, self->priv->_display);
	net_set_redirection_type (self->priv->net, NET_REDIRECTION_TYPE_NONE);
	g_signal_connect ((GObject*) self->priv->net, "notify::url", (GCallback) __lambda9__g_object_notify, self);
	g_signal_connect ((GObject*) self->priv->net, "notify::redirection-type", (GCallback) __lambda10__g_object_notify, self);
	g_signal_connect ((GObject*) self->priv->net, "notify::redirection-status", (GCallback) __lambda11__g_object_notify, self);
}


static void meiga_server_initialize_mimetypes (MeigaServer* self) {
	GError * _inner_error_;
	GHashTable* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	_tmp0_ = NULL;
	self->priv->mimetypes = (_tmp0_ = g_hash_table_new (g_str_hash, g_str_equal), (self->priv->mimetypes == NULL) ? NULL : (self->priv->mimetypes = (g_hash_table_unref (self->priv->mimetypes), NULL)), _tmp0_);
	{
		FILE* f;
		gchar* _tmp1_;
		gint buffline_size;
		gint buffline_length1;
		gchar* buffline;
		char* line;
		f = fopen (MEIGA_SERVER_MIME_TYPES_FILE, "r");
		_tmp1_ = NULL;
		buffline = (_tmp1_ = g_new0 (gchar, 128), buffline_length1 = 128, buffline_size = buffline_length1, _tmp1_);
		line = NULL;
		if (f == NULL) {
			_inner_error_ = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_FAILED, "opening");
			if (_inner_error_ != NULL) {
				(f == NULL) ? NULL : (f = (fclose (f), NULL));
				buffline = (g_free (buffline), NULL);
				line = (g_free (line), NULL);
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
		}
		while (TRUE) {
			char* _tmp3_;
			const char* _tmp2_;
			gboolean _tmp4_;
			char** _tmp6_;
			gint buf_size;
			gint buf_length1;
			char** _tmp5_;
			char** buf;
			const char* _tmp7_;
			char* mime;
			char** _tmp9_;
			gint extension_size;
			gint extension_length1;
			char** _tmp8_;
			char** extension;
			if (!(!feof (f))) {
				break;
			}
			fgets (buffline, buffline_length1, f);
			_tmp3_ = NULL;
			_tmp2_ = NULL;
			line = (_tmp3_ = (_tmp2_ = (const char*) buffline, (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_)), line = (g_free (line), NULL), _tmp3_);
			_tmp4_ = FALSE;
			if (line == NULL) {
				_tmp4_ = TRUE;
			} else {
				_tmp4_ = feof (f);
			}
			if (_tmp4_) {
				continue;
			}
			_tmp6_ = NULL;
			_tmp5_ = NULL;
			buf = (_tmp6_ = _tmp5_ = g_strsplit_set (line, " \t", 2), buf_length1 = _vala_array_length (_tmp5_), buf_size = buf_length1, _tmp6_);
			_tmp7_ = NULL;
			mime = (_tmp7_ = g_strstrip (buf[0]), (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_));
			if (buf[1] == NULL) {
				buf = (_vala_array_free (buf, buf_length1, (GDestroyNotify) g_free), NULL);
				mime = (g_free (mime), NULL);
				continue;
			}
			_tmp9_ = NULL;
			_tmp8_ = NULL;
			extension = (_tmp9_ = _tmp8_ = g_strsplit_set (g_strstrip (buf[1]), " \t", 0), extension_length1 = _vala_array_length (_tmp8_), extension_size = extension_length1, _tmp9_);
			{
				gint i;
				i = 0;
				{
					gboolean _tmp10_;
					_tmp10_ = TRUE;
					while (TRUE) {
						const char* _tmp12_;
						const char* _tmp11_;
						if (!_tmp10_) {
							i++;
						}
						_tmp10_ = FALSE;
						if (!(extension[i] != NULL)) {
							break;
						}
						_tmp12_ = NULL;
						_tmp11_ = NULL;
						g_hash_table_insert (self->priv->mimetypes, (_tmp11_ = g_strstrip (extension[i]), (_tmp11_ == NULL) ? NULL : g_strdup (_tmp11_)), (_tmp12_ = mime, (_tmp12_ == NULL) ? NULL : g_strdup (_tmp12_)));
					}
				}
			}
			buf = (_vala_array_free (buf, buf_length1, (GDestroyNotify) g_free), NULL);
			mime = (g_free (mime), NULL);
			extension = (_vala_array_free (extension, extension_length1, (GDestroyNotify) g_free), NULL);
		}
		(f == NULL) ? NULL : (f = (fclose (f), NULL));
		buffline = (g_free (buffline), NULL);
		line = (g_free (line), NULL);
	}
	goto __finally0;
	__catch0_g_file_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp13_;
			_tmp13_ = NULL;
			meiga_server_log (self, _tmp13_ = g_strdup_printf (_ ("Error opening file %s"), MEIGA_SERVER_MIME_TYPES_FILE));
			_tmp13_ = (g_free (_tmp13_), NULL);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


guint _dynamic_request_name0 (DBusGProxy* self, const char* param1, guint param2, GError** error) {
	guint result;
	dbus_g_proxy_call (self, "RequestName", error, G_TYPE_STRING, param1, G_TYPE_UINT, param2, G_TYPE_INVALID, G_TYPE_UINT, &result, G_TYPE_INVALID);
	if (*error) {
		return 0U;
	}
	return result;
}


static void meiga_server_initialize_dbus (MeigaServer* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	{
		Meiga* _tmp0_;
		DBusGConnection* conn;
		DBusGProxy* bus;
		guint request_name_result;
		_tmp0_ = NULL;
		self->priv->exposed = (_tmp0_ = meiga_new (self), (self->priv->exposed == NULL) ? NULL : (self->priv->exposed = (g_object_unref (self->priv->exposed), NULL)), _tmp0_);
		conn = dbus_g_bus_get (DBUS_BUS_SESSION, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch1_g_error;
			goto __finally1;
		}
		bus = dbus_g_proxy_new_for_name (conn, "org.freedesktop.DBus", "/org/freedesktop/DBus", "org.freedesktop.DBus");
		request_name_result = _dynamic_request_name0 (bus, "com.igalia.Meiga", (guint) 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
			(bus == NULL) ? NULL : (bus = (g_object_unref (bus), NULL));
			goto __catch1_g_error;
			goto __finally1;
		}
		if (request_name_result == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
			meiga_server_log (self, _ ("Registering DBUS service"));
			_vala_dbus_register_object (dbus_g_connection_get_connection (conn), "/com/igalia/Meiga", G_OBJECT (self->priv->exposed));
		} else {
			meiga_server_log (self, _ ("Not registering DBUS service: not primary owner"));
		}
		(conn == NULL) ? NULL : (conn = (dbus_g_connection_unref (conn), NULL));
		(bus == NULL) ? NULL : (bus = (g_object_unref (bus), NULL));
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp1_;
			_tmp1_ = NULL;
			meiga_server_log (self, _tmp1_ = g_strdup_printf (_ ("Error registering DBUS server: %s"), e->message));
			_tmp1_ = (g_free (_tmp1_), NULL);
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


char* meiga_server_get_public_url (MeigaServer* self) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = net_get_url (self->priv->net);
	return result;
}


static gboolean _lambda12_ (MeigaServer* self) {
	gboolean result;
	gtk_main_quit ();
	result = FALSE;
	return result;
}


static gboolean __lambda12__gsource_func (gpointer self) {
	return _lambda12_ (self);
}


void meiga_server_shutdown (MeigaServer* self) {
	g_return_if_fail (self != NULL);
	net_set_redirection_type (self->priv->net, NET_REDIRECTION_TYPE_NONE);
	g_idle_add_full (G_PRIORITY_LOW, __lambda12__gsource_func, g_object_ref (self), g_object_unref);
}


void meiga_server_set_port (MeigaServer* self, gint port) {
	g_return_if_fail (self != NULL);
	self->priv->port = port;
}


void meiga_server_register_gui (MeigaServer* self, guint gui_pid, const char* display) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (display != NULL);
	meiga_server_set_gui_pid (self, gui_pid);
	meiga_server_set_display (self, display);
	if (self->priv->net == NULL) {
		meiga_server_initialize_net (self);
	}
}


static void _meiga_server_serve_file_callback_soup_server_callback (SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client, gpointer self) {
	meiga_server_serve_file_callback (self, server, msg, path, query, client);
}


void meiga_server_register_path (MeigaServer* self, const char* real_path, const char* logical_path) {
	const char* _tmp1_;
	const char* _tmp0_;
	char* _tmp2_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (real_path != NULL);
	g_return_if_fail (logical_path != NULL);
	/* For security reasons, don't allow path registering if
	 there's no GUI to monitor it*/
	if (self->priv->_gui_pid == 0) {
		meiga_server_log (self, _ ("Attempted to register path without GUI started. Ignoring."));
		return;
	}
	if (((const char*) g_hash_table_lookup (self->priv->path_mapping, logical_path)) != NULL) {
		return;
	}
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	g_hash_table_insert (self->priv->path_mapping, (_tmp0_ = logical_path, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), (_tmp1_ = real_path, (_tmp1_ == NULL) ? NULL : g_strdup (_tmp1_)));
	_tmp2_ = NULL;
	meiga_server_log (self, _tmp2_ = g_strdup_printf (_ ("Registered logical path '%s' to real path '%s'"), logical_path, real_path));
	_tmp2_ = (g_free (_tmp2_), NULL);
	soup_server_add_handler (self->priv->server, logical_path, _meiga_server_serve_file_callback_soup_server_callback, g_object_ref (self), g_object_unref);
}


void meiga_server_unregister_path (MeigaServer* self, const char* logical_path) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (logical_path != NULL);
	/* For security reasons, don't allow path unregistering if
	 there's no GUI to monitor it*/
	if (self->priv->_gui_pid == 0) {
		meiga_server_log (self, _ ("Attempted to unregister path without GUI started. Ignoring."));
		return;
	}
	_tmp0_ = NULL;
	meiga_server_log (self, _tmp0_ = g_strdup_printf (_ ("Unregistered logical path '%s'"), logical_path));
	_tmp0_ = (g_free (_tmp0_), NULL);
	g_hash_table_remove (self->priv->path_mapping, logical_path);
}


gint meiga_server_get_redirection_type (MeigaServer* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	/* GUI should have registered itself before changing redirection type*/
	if (self->priv->net == NULL) {
		result = NET_REDIRECTION_TYPE_NONE;
		return result;
	} else {
		result = net_get_redirection_type (self->priv->net);
		return result;
	}
}


void meiga_server_set_redirection_type (MeigaServer* self, gint redirection_type) {
	g_return_if_fail (self != NULL);
	/* GUI should have registered itself before changing redirection type*/
	if (self->priv->net == NULL) {
		return;
	}
	net_set_redirection_type (self->priv->net, redirection_type);
}


gint meiga_server_get_redirection_status (MeigaServer* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	/* GUI should have registered itself before changing redirection status*/
	if (self->priv->net == NULL) {
		result = NET_REDIRECTION_STATUS_NONE;
		return result;
	} else {
		result = net_get_redirection_status (self->priv->net);
		return result;
	}
}


void meiga_server_set_ssh_host (MeigaServer* self, const char* ssh_host) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_host != NULL);
	/* GUI should have registered itself before doing changes*/
	if (self->priv->net == NULL) {
		return;
	}
	net_set_ssh_host (self->priv->net, ssh_host);
}


void meiga_server_set_ssh_port (MeigaServer* self, const char* ssh_port) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_port != NULL);
	/* GUI should have registered itself before doing changes*/
	if (self->priv->net == NULL) {
		return;
	}
	net_set_ssh_port (self->priv->net, ssh_port);
}


void meiga_server_set_ssh_user (MeigaServer* self, const char* ssh_user) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_user != NULL);
	/* GUI should have registered itself before doing changes*/
	if (self->priv->net == NULL) {
		return;
	}
	net_set_ssh_user (self->priv->net, ssh_user);
}


void meiga_server_set_ssh_password (MeigaServer* self, const char* ssh_password) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ssh_password != NULL);
	/* GUI should have registered itself before doing changes*/
	if (self->priv->net == NULL) {
		return;
	}
	net_set_ssh_password (self->priv->net, ssh_password);
}


GHashTable* meiga_server_get_paths (MeigaServer* self) {
	GHashTable* result;
	GHashTable* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	result = (_tmp0_ = self->priv->path_mapping, (_tmp0_ == NULL) ? NULL : g_hash_table_ref (_tmp0_));
	return result;
}


char* meiga_server_get_requests_stats (MeigaServer* self) {
	char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = g_strdup_printf ("%u/%u", self->priv->pending_requests, self->priv->total_requests);
	return result;
}


static void _lambda2_ (MeigaServer* self) {
	self->priv->pending_requests--;
	g_signal_emit_by_name (self, "model-changed");
}


static void __lambda2__soup_message_finished (SoupMessage* _sender, gpointer self) {
	_lambda2_ (self);
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


void meiga_server_serve_file_callback (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client) {
	GError * _inner_error_;
	char* real_path;
	char** _tmp1_;
	gint path_tokens_size;
	gint path_tokens_length1;
	char** _tmp0_;
	char** path_tokens;
	char* composed_path;
	char* translated_path;
	gint j;
	char* _tmp12_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (server != NULL);
	g_return_if_fail (path != NULL);
	_inner_error_ = NULL;
	real_path = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	path_tokens = (_tmp1_ = _tmp0_ = g_strsplit (path, "/", 32), path_tokens_length1 = _vala_array_length (_tmp0_), path_tokens_size = path_tokens_length1, _tmp1_);
	composed_path = g_strdup ("");
	translated_path = g_strdup ("");
	j = 0;
	{
		gint i;
		i = 0;
		{
			gboolean _tmp2_;
			_tmp2_ = TRUE;
			while (TRUE) {
				char* _tmp4_;
				char* _tmp3_;
				char* _tmp6_;
				const char* _tmp5_;
				if (!_tmp2_) {
					i++;
				}
				_tmp2_ = FALSE;
				if (!(path_tokens[i] != NULL)) {
					break;
				}
				if (_vala_strcmp0 (path_tokens[i], "") == 0) {
					continue;
				}
				_tmp4_ = NULL;
				_tmp3_ = NULL;
				composed_path = (_tmp4_ = g_strconcat (composed_path, _tmp3_ = g_strconcat ("/", path_tokens[i], NULL), NULL), composed_path = (g_free (composed_path), NULL), _tmp4_);
				_tmp3_ = (g_free (_tmp3_), NULL);
				_tmp6_ = NULL;
				_tmp5_ = NULL;
				translated_path = (_tmp6_ = (_tmp5_ = (const char*) g_hash_table_lookup (self->priv->path_mapping, composed_path), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), translated_path = (g_free (translated_path), NULL), _tmp6_);
				if (translated_path != NULL) {
					char* _tmp8_;
					const char* _tmp7_;
					_tmp8_ = NULL;
					_tmp7_ = NULL;
					real_path = (_tmp8_ = (_tmp7_ = translated_path, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), real_path = (g_free (real_path), NULL), _tmp8_);
					j = i + 1;
				}
			}
		}
	}
	{
		gint i;
		i = j;
		{
			gboolean _tmp9_;
			_tmp9_ = TRUE;
			while (TRUE) {
				char* _tmp11_;
				char* _tmp10_;
				if (!_tmp9_) {
					i++;
				}
				_tmp9_ = FALSE;
				if (!(path_tokens[i] != NULL)) {
					break;
				}
				_tmp11_ = NULL;
				_tmp10_ = NULL;
				real_path = (_tmp11_ = g_strconcat (real_path, _tmp10_ = g_strconcat ("/", path_tokens[i], NULL), NULL), real_path = (g_free (real_path), NULL), _tmp11_);
				_tmp10_ = (g_free (_tmp10_), NULL);
			}
		}
	}
	self->priv->pending_requests++;
	self->priv->total_requests++;
	g_signal_emit_by_name (self, "model-changed");
	g_signal_connect (msg, "finished", (GCallback) __lambda2__soup_message_finished, self);
	_tmp12_ = NULL;
	meiga_server_log (self, _tmp12_ = g_strdup_printf (_ ("Request: %s --> Serving: %s"), path, real_path));
	_tmp12_ = (g_free (_tmp12_), NULL);
	if (real_path == NULL) {
		meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
		real_path = (g_free (real_path), NULL);
		path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
		composed_path = (g_free (composed_path), NULL);
		translated_path = (g_free (translated_path), NULL);
		return;
	} else {
		if (!g_file_test (real_path, G_FILE_TEST_EXISTS)) {
			meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
			real_path = (g_free (real_path), NULL);
			path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
			composed_path = (g_free (composed_path), NULL);
			translated_path = (g_free (translated_path), NULL);
			return;
		} else {
			if (g_file_test (real_path, G_FILE_TEST_IS_REGULAR)) {
				GMappedFile* f;
				char* _tmp15_;
				char* _tmp16_;
				char* extension;
				char* mime;
				f = NULL;
				{
					GMappedFile* _tmp13_;
					GMappedFile* _tmp14_;
					_tmp13_ = g_mapped_file_new (real_path, FALSE, &_inner_error_);
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_FILE_ERROR) {
							goto __catch2_g_file_error;
						}
						goto __finally2;
					}
					_tmp14_ = NULL;
					f = (_tmp14_ = _tmp13_, (f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL)), _tmp14_);
				}
				goto __finally2;
				__catch2_g_file_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
						meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
						real_path = (g_free (real_path), NULL);
						path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
						composed_path = (g_free (composed_path), NULL);
						translated_path = (g_free (translated_path), NULL);
						return;
					}
				}
				__finally2:
				if (_inner_error_ != NULL) {
					(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
					real_path = (g_free (real_path), NULL);
					path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
					composed_path = (g_free (composed_path), NULL);
					translated_path = (g_free (translated_path), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				soup_message_set_status (msg, (guint) SOUP_STATUS_OK);
				_tmp15_ = NULL;
				_tmp16_ = NULL;
				extension = (_tmp16_ = g_utf8_strdown (_tmp15_ = meiga_server_extension_from_path (self, real_path), -1), _tmp15_ = (g_free (_tmp15_), NULL), _tmp16_);
				mime = NULL;
				if (extension != NULL) {
					char* _tmp18_;
					const char* _tmp17_;
					_tmp18_ = NULL;
					_tmp17_ = NULL;
					mime = (_tmp18_ = (_tmp17_ = (const char*) g_hash_table_lookup (self->priv->mimetypes, extension), (_tmp17_ == NULL) ? NULL : g_strdup (_tmp17_)), mime = (g_free (mime), NULL), _tmp18_);
				}
				if (mime == NULL) {
					char* _tmp19_;
					_tmp19_ = NULL;
					mime = (_tmp19_ = g_strdup ("application/x-octet-stream"), mime = (g_free (mime), NULL), _tmp19_);
				}
				/* f.get_contents() returns unmanaged memory, so Vala will segfault trying to
				 unref a normal string. AN unmanaged string shoyuld be used (string *)*/
				soup_message_set_response (msg, mime, SOUP_MEMORY_COPY, (const char*) g_mapped_file_get_contents (f), (gsize) g_mapped_file_get_length (f));
				(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
				extension = (g_free (extension), NULL);
				mime = (g_free (mime), NULL);
			} else {
				if (g_file_test (real_path, G_FILE_TEST_IS_DIR)) {
					GList* files;
					GDir* d;
					char* response;
					char* _tmp27_;
					char* _tmp28_;
					char* _tmp29_;
					char* _tmp30_;
					char* _tmp32_;
					char* _tmp31_;
					char* _tmp33_;
					char* _tmp34_;
					char* _tmp35_;
					char* _tmp37_;
					char* _tmp36_;
					char* _tmp39_;
					char* _tmp38_;
					char* _tmp40_;
					char* _tmp49_;
					char* _tmp50_;
					char* _tmp52_;
					char* _tmp51_;
					char* _tmp53_;
					char* _tmp54_;
					files = NULL;
					d = NULL;
					{
						GDir* _tmp20_;
						GDir* _tmp21_;
						_tmp20_ = g_dir_open (real_path, (guint) 0, &_inner_error_);
						if (_inner_error_ != NULL) {
							if (_inner_error_->domain == G_FILE_ERROR) {
								goto __catch3_g_file_error;
							}
							goto __finally3;
						}
						_tmp21_ = NULL;
						d = (_tmp21_ = _tmp20_, (d == NULL) ? NULL : (d = (g_dir_close (d), NULL)), _tmp21_);
						{
							const char* _tmp22_;
							char* f;
							_tmp22_ = NULL;
							f = (_tmp22_ = g_dir_read_name (d), (_tmp22_ == NULL) ? NULL : g_strdup (_tmp22_));
							{
								gboolean _tmp23_;
								_tmp23_ = TRUE;
								while (TRUE) {
									const char* _tmp26_;
									if (!_tmp23_) {
										char* _tmp25_;
										const char* _tmp24_;
										_tmp25_ = NULL;
										_tmp24_ = NULL;
										f = (_tmp25_ = (_tmp24_ = g_dir_read_name (d), (_tmp24_ == NULL) ? NULL : g_strdup (_tmp24_)), f = (g_free (f), NULL), _tmp25_);
									}
									_tmp23_ = FALSE;
									if (!(f != NULL)) {
										break;
									}
									_tmp26_ = NULL;
									files = g_list_insert_sorted (files, (_tmp26_ = f, (_tmp26_ == NULL) ? NULL : g_strdup (_tmp26_)), strcmp);
								}
							}
							f = (g_free (f), NULL);
						}
					}
					goto __finally3;
					__catch3_g_file_error:
					{
						GError * e;
						e = _inner_error_;
						_inner_error_ = NULL;
						{
							meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
							(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
							(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
							(d == NULL) ? NULL : (d = (g_dir_close (d), NULL));
							real_path = (g_free (real_path), NULL);
							path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
							composed_path = (g_free (composed_path), NULL);
							translated_path = (g_free (translated_path), NULL);
							return;
						}
					}
					__finally3:
					if (_inner_error_ != NULL) {
						(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
						(d == NULL) ? NULL : (d = (g_dir_close (d), NULL));
						real_path = (g_free (real_path), NULL);
						path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
						composed_path = (g_free (composed_path), NULL);
						translated_path = (g_free (translated_path), NULL);
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return;
					}
					soup_message_set_status (msg, (guint) SOUP_STATUS_OK);
					response = g_strdup ("");
					_tmp27_ = NULL;
					response = (_tmp27_ = g_strconcat (response, "<html>\n", NULL), response = (g_free (response), NULL), _tmp27_);
					_tmp28_ = NULL;
					response = (_tmp28_ = g_strconcat (response, "<head>\n", NULL), response = (g_free (response), NULL), _tmp28_);
					_tmp29_ = NULL;
					response = (_tmp29_ = g_strconcat (response, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n", NULL), response = (g_free (response), NULL), _tmp29_);
					_tmp30_ = NULL;
					response = (_tmp30_ = g_strconcat (response, "<title>\n", NULL), response = (g_free (response), NULL), _tmp30_);
					_tmp32_ = NULL;
					_tmp31_ = NULL;
					response = (_tmp32_ = g_strconcat (response, _tmp31_ = g_strdup_printf (_ ("Index of %s\n"), path), NULL), response = (g_free (response), NULL), _tmp32_);
					_tmp31_ = (g_free (_tmp31_), NULL);
					_tmp33_ = NULL;
					response = (_tmp33_ = g_strconcat (response, "</title>\n", NULL), response = (g_free (response), NULL), _tmp33_);
					_tmp34_ = NULL;
					response = (_tmp34_ = g_strconcat (response, "</head>\n", NULL), response = (g_free (response), NULL), _tmp34_);
					_tmp35_ = NULL;
					response = (_tmp35_ = g_strconcat (response, "<body>\n", NULL), response = (g_free (response), NULL), _tmp35_);
					_tmp37_ = NULL;
					_tmp36_ = NULL;
					response = (_tmp37_ = g_strconcat (response, _tmp36_ = g_strdup_printf ("<!-- generator=\"%s/%s\" -->\n", PACKAGE, VERSION), NULL), response = (g_free (response), NULL), _tmp37_);
					_tmp36_ = (g_free (_tmp36_), NULL);
					_tmp39_ = NULL;
					_tmp38_ = NULL;
					response = (_tmp39_ = g_strconcat (response, _tmp38_ = g_strdup_printf (_ ("Index of %s\n"), path), NULL), response = (g_free (response), NULL), _tmp39_);
					_tmp38_ = (g_free (_tmp38_), NULL);
					_tmp40_ = NULL;
					response = (_tmp40_ = g_strconcat (response, "<ul>\n", NULL), response = (g_free (response), NULL), _tmp40_);
					{
						GList* f_collection;
						GList* f_it;
						f_collection = files;
						for (f_it = f_collection; f_it != NULL; f_it = f_it->next) {
							const char* _tmp48_;
							char* f;
							_tmp48_ = NULL;
							f = (_tmp48_ = (const char*) f_it->data, (_tmp48_ == NULL) ? NULL : g_strdup (_tmp48_));
							{
								char* _tmp47_;
								char* _tmp46_;
								char* _tmp45_;
								char* _tmp44_;
								char* _tmp43_;
								char* _tmp42_;
								char* _tmp41_;
								_tmp47_ = NULL;
								_tmp46_ = NULL;
								_tmp45_ = NULL;
								_tmp44_ = NULL;
								_tmp43_ = NULL;
								_tmp42_ = NULL;
								_tmp41_ = NULL;
								response = (_tmp47_ = g_strconcat (response, _tmp46_ = g_strconcat (_tmp45_ = g_strconcat (_tmp44_ = g_strconcat (_tmp43_ = g_strconcat (_tmp42_ = g_strconcat (_tmp41_ = g_strconcat ("<li><a href=\"", path, NULL), "/", NULL), f, NULL), "\">", NULL), f, NULL), "</a></li>", NULL), NULL), response = (g_free (response), NULL), _tmp47_);
								_tmp46_ = (g_free (_tmp46_), NULL);
								_tmp45_ = (g_free (_tmp45_), NULL);
								_tmp44_ = (g_free (_tmp44_), NULL);
								_tmp43_ = (g_free (_tmp43_), NULL);
								_tmp42_ = (g_free (_tmp42_), NULL);
								_tmp41_ = (g_free (_tmp41_), NULL);
								f = (g_free (f), NULL);
							}
						}
					}
					_tmp49_ = NULL;
					response = (_tmp49_ = g_strconcat (response, "</ul>\n", NULL), response = (g_free (response), NULL), _tmp49_);
					_tmp50_ = NULL;
					response = (_tmp50_ = g_strconcat (response, "<br/><br/><i>", NULL), response = (g_free (response), NULL), _tmp50_);
					_tmp52_ = NULL;
					_tmp51_ = NULL;
					response = (_tmp52_ = g_strconcat (response, _tmp51_ = g_strdup_printf (_ ("Served by <a href=\"http://meiga.igalia.com\">Meiga %s</a>"), VERSION), NULL), response = (g_free (response), NULL), _tmp52_);
					_tmp51_ = (g_free (_tmp51_), NULL);
					_tmp53_ = NULL;
					response = (_tmp53_ = g_strconcat (response, "</i>\n", NULL), response = (g_free (response), NULL), _tmp53_);
					_tmp54_ = NULL;
					response = (_tmp54_ = g_strconcat (response, "</body>\n</html>\n", NULL), response = (g_free (response), NULL), _tmp54_);
					soup_message_set_response (msg, "text/html", SOUP_MEMORY_COPY, response, (gsize) g_utf8_strlen (response, -1));
					(files == NULL) ? NULL : (files = (_g_list_free_g_free (files), NULL));
					(d == NULL) ? NULL : (d = (g_dir_close (d), NULL));
					response = (g_free (response), NULL);
				} else {
					meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
				}
			}
		}
	}
	real_path = (g_free (real_path), NULL);
	path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
	composed_path = (g_free (composed_path), NULL);
	translated_path = (g_free (translated_path), NULL);
}


void meiga_server_serve_rss_callback (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client) {
	GError * _inner_error_;
	char* real_path;
	char** _tmp1_;
	gint path_tokens_size;
	gint path_tokens_length1;
	char** _tmp0_;
	char** path_tokens;
	char* composed_path;
	char* translated_path;
	gint j;
	char* _tmp12_;
	SoupURI* uri;
	char* base_url;
	g_return_if_fail (self != NULL);
	g_return_if_fail (server != NULL);
	g_return_if_fail (path != NULL);
	_inner_error_ = NULL;
	real_path = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	path_tokens = (_tmp1_ = _tmp0_ = g_strsplit (path, "/", 32), path_tokens_length1 = _vala_array_length (_tmp0_), path_tokens_size = path_tokens_length1, _tmp1_);
	composed_path = g_strdup ("");
	translated_path = g_strdup ("");
	j = 0;
	{
		gint i;
		/* Ignoring first path element "/rss"*/
		i = 2;
		{
			gboolean _tmp2_;
			_tmp2_ = TRUE;
			while (TRUE) {
				char* _tmp4_;
				char* _tmp3_;
				char* _tmp6_;
				const char* _tmp5_;
				if (!_tmp2_) {
					i++;
				}
				_tmp2_ = FALSE;
				if (!(path_tokens[i] != NULL)) {
					break;
				}
				if (_vala_strcmp0 (path_tokens[i], "") == 0) {
					continue;
				}
				_tmp4_ = NULL;
				_tmp3_ = NULL;
				composed_path = (_tmp4_ = g_strconcat (composed_path, _tmp3_ = g_strconcat ("/", path_tokens[i], NULL), NULL), composed_path = (g_free (composed_path), NULL), _tmp4_);
				_tmp3_ = (g_free (_tmp3_), NULL);
				_tmp6_ = NULL;
				_tmp5_ = NULL;
				translated_path = (_tmp6_ = (_tmp5_ = (const char*) g_hash_table_lookup (self->priv->path_mapping, composed_path), (_tmp5_ == NULL) ? NULL : g_strdup (_tmp5_)), translated_path = (g_free (translated_path), NULL), _tmp6_);
				if (translated_path != NULL) {
					char* _tmp8_;
					const char* _tmp7_;
					_tmp8_ = NULL;
					_tmp7_ = NULL;
					real_path = (_tmp8_ = (_tmp7_ = translated_path, (_tmp7_ == NULL) ? NULL : g_strdup (_tmp7_)), real_path = (g_free (real_path), NULL), _tmp8_);
					j = i + 1;
				}
			}
		}
	}
	{
		gint i;
		i = j;
		{
			gboolean _tmp9_;
			_tmp9_ = TRUE;
			while (TRUE) {
				char* _tmp11_;
				char* _tmp10_;
				if (!_tmp9_) {
					i++;
				}
				_tmp9_ = FALSE;
				if (!(path_tokens[i] != NULL)) {
					break;
				}
				_tmp11_ = NULL;
				_tmp10_ = NULL;
				real_path = (_tmp11_ = g_strconcat (real_path, _tmp10_ = g_strconcat ("/", path_tokens[i], NULL), NULL), real_path = (g_free (real_path), NULL), _tmp11_);
				_tmp10_ = (g_free (_tmp10_), NULL);
			}
		}
	}
	_tmp12_ = NULL;
	meiga_server_log (self, _tmp12_ = g_strdup_printf (_ ("Request: %s --> Serving (RSS mode): %s"), path, real_path));
	_tmp12_ = (g_free (_tmp12_), NULL);
	uri = soup_uri_copy (soup_message_get_uri (msg));
	base_url = g_strdup_printf ("%s://%s:%u", uri->scheme, uri->host, uri->port);
	if (real_path == NULL) {
		meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
		real_path = (g_free (real_path), NULL);
		path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
		composed_path = (g_free (composed_path), NULL);
		translated_path = (g_free (translated_path), NULL);
		(uri == NULL) ? NULL : (uri = (soup_uri_free (uri), NULL));
		base_url = (g_free (base_url), NULL);
		return;
	} else {
		if (!g_file_test (real_path, G_FILE_TEST_EXISTS)) {
			meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
			real_path = (g_free (real_path), NULL);
			path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
			composed_path = (g_free (composed_path), NULL);
			translated_path = (g_free (translated_path), NULL);
			(uri == NULL) ? NULL : (uri = (soup_uri_free (uri), NULL));
			base_url = (g_free (base_url), NULL);
			return;
		} else {
			if (g_file_test (real_path, G_FILE_TEST_IS_REGULAR)) {
				GMappedFile* f;
				char* extension;
				char* mime;
				f = NULL;
				{
					GMappedFile* _tmp13_;
					GMappedFile* _tmp14_;
					_tmp13_ = g_mapped_file_new (real_path, FALSE, &_inner_error_);
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_FILE_ERROR) {
							goto __catch4_g_file_error;
						}
						goto __finally4;
					}
					_tmp14_ = NULL;
					f = (_tmp14_ = _tmp13_, (f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL)), _tmp14_);
				}
				goto __finally4;
				__catch4_g_file_error:
				{
					GError * e;
					e = _inner_error_;
					_inner_error_ = NULL;
					{
						meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
						(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
						(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
						real_path = (g_free (real_path), NULL);
						path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
						composed_path = (g_free (composed_path), NULL);
						translated_path = (g_free (translated_path), NULL);
						(uri == NULL) ? NULL : (uri = (soup_uri_free (uri), NULL));
						base_url = (g_free (base_url), NULL);
						return;
					}
				}
				__finally4:
				if (_inner_error_ != NULL) {
					(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
					real_path = (g_free (real_path), NULL);
					path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
					composed_path = (g_free (composed_path), NULL);
					translated_path = (g_free (translated_path), NULL);
					(uri == NULL) ? NULL : (uri = (soup_uri_free (uri), NULL));
					base_url = (g_free (base_url), NULL);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				soup_message_set_status (msg, (guint) SOUP_STATUS_OK);
				extension = meiga_server_extension_from_path (self, real_path);
				mime = NULL;
				if (extension != NULL) {
					char* _tmp16_;
					const char* _tmp15_;
					_tmp16_ = NULL;
					_tmp15_ = NULL;
					mime = (_tmp16_ = (_tmp15_ = (const char*) g_hash_table_lookup (self->priv->mimetypes, extension), (_tmp15_ == NULL) ? NULL : g_strdup (_tmp15_)), mime = (g_free (mime), NULL), _tmp16_);
				}
				if (mime == NULL) {
					char* _tmp17_;
					_tmp17_ = NULL;
					mime = (_tmp17_ = g_strdup ("application/x-octet-stream"), mime = (g_free (mime), NULL), _tmp17_);
				}
				/* f.get_contents() returns unmanaged memory, so Vala will segfault trying to
				 unref a normal string. AN unmanaged string shoyuld be used (string *)*/
				soup_message_set_response (msg, mime, SOUP_MEMORY_COPY, (const char*) g_mapped_file_get_contents (f), (gsize) g_mapped_file_get_length (f));
				(f == NULL) ? NULL : (f = (g_mapped_file_free (f), NULL));
				extension = (g_free (extension), NULL);
				mime = (g_free (mime), NULL);
			} else {
				if (g_file_test (real_path, G_FILE_TEST_IS_DIR)) {
					char* _tmp19_;
					char* _tmp18_;
					RssFeed* _tmp20_;
					RssFeed* rss;
					char* response;
					soup_message_set_status (msg, (guint) SOUP_STATUS_OK);
					_tmp19_ = NULL;
					_tmp18_ = NULL;
					_tmp20_ = NULL;
					rss = (_tmp20_ = rss_feed_new_from_directory (real_path, _tmp18_ = g_strconcat (base_url, composed_path, NULL), composed_path, _tmp19_ = g_strdup_printf (_ ("Documents under %s"), composed_path)), _tmp19_ = (g_free (_tmp19_), NULL), _tmp18_ = (g_free (_tmp18_), NULL), _tmp20_);
					response = rss_feed_to_string (rss);
					soup_message_set_response (msg, "application/rss+xml", SOUP_MEMORY_COPY, response, (gsize) g_utf8_strlen (response, -1));
					(rss == NULL) ? NULL : (rss = (g_object_unref (rss), NULL));
					response = (g_free (response), NULL);
				} else {
					meiga_server_serve_file_callback_default (self, server, msg, path, query, client);
				}
			}
		}
	}
	real_path = (g_free (real_path), NULL);
	path_tokens = (_vala_array_free (path_tokens, path_tokens_length1, (GDestroyNotify) g_free), NULL);
	composed_path = (g_free (composed_path), NULL);
	translated_path = (g_free (translated_path), NULL);
	(uri == NULL) ? NULL : (uri = (soup_uri_free (uri), NULL));
	base_url = (g_free (base_url), NULL);
}


void meiga_server_serve_file_callback_default (MeigaServer* self, SoupServer* server, SoupMessage* msg, const char* path, GHashTable* query, SoupClientContext* client) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (server != NULL);
	g_return_if_fail (msg != NULL);
	g_return_if_fail (path != NULL);
	meiga_server_serve_error (self, msg, path, SOUP_STATUS_NOT_FOUND, _ ("File not found"));
}


void meiga_server_serve_error (MeigaServer* self, SoupMessage* msg, const char* path, SoupKnownStatusCode _error_, const char* error_message) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (msg != NULL);
	g_return_if_fail (path != NULL);
	g_return_if_fail (error_message != NULL);
	soup_message_set_status (msg, (guint) _error_);
	soup_message_set_response (msg, "text/html", SOUP_MEMORY_COPY, error_message, (gsize) g_utf8_strlen (error_message, -1));
	_tmp0_ = NULL;
	meiga_server_log (self, _tmp0_ = g_strdup_printf ("%d - %s: %s", (gint) _error_, error_message, path));
	_tmp0_ = (g_free (_tmp0_), NULL);
}


static char* meiga_server_extension_from_path (MeigaServer* self, const char* path) {
	char* result;
	char** _tmp1_;
	gint token_size;
	gint token_length1;
	char** _tmp0_;
	char** token;
	gint i;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	token = (_tmp1_ = _tmp0_ = g_strsplit (path, ".", 0), token_length1 = _vala_array_length (_tmp0_), token_size = token_length1, _tmp1_);
	i = 0;
	while (TRUE) {
		if (!(token[i] != NULL)) {
			break;
		}
		i++;
	}
	if (i > 0) {
		const char* _tmp2_;
		_tmp2_ = NULL;
		result = (_tmp2_ = token[i - 1], (_tmp2_ == NULL) ? NULL : g_strdup (_tmp2_));
		token = (_vala_array_free (token, token_length1, (GDestroyNotify) g_free), NULL);
		return result;
	} else {
		result = NULL;
		token = (_vala_array_free (token, token_length1, (GDestroyNotify) g_free), NULL);
		return result;
	}
	token = (_vala_array_free (token, token_length1, (GDestroyNotify) g_free), NULL);
}


void meiga_server_main (char** args, int args_length1) {
	MeigaServer* s;
	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
	s = meiga_server_new ();
	meiga_server_initialize (s);
	gtk_init (&args_length1, &args);
	gtk_main ();
	(s == NULL) ? NULL : (s = (g_object_unref (s), NULL));
	return;
}


int main (int argc, char ** argv) {
	g_thread_init (NULL);
	g_type_init ();
	meiga_server_main (argv, argc);
	return 0;
}


Log* meiga_server_get_logger (MeigaServer* self) {
	Log* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_logger;
	return result;
}


static void meiga_server_set_logger (MeigaServer* self, Log* value) {
	Log* _tmp1_;
	Log* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_logger = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_object_ref (_tmp0_)), (self->priv->_logger == NULL) ? NULL : (self->priv->_logger = (g_object_unref (self->priv->_logger), NULL)), _tmp1_);
	g_object_notify ((GObject *) self, "logger");
}


guint meiga_server_get_gui_pid (MeigaServer* self) {
	guint result;
	g_return_val_if_fail (self != NULL, 0U);
	result = self->priv->_gui_pid;
	return result;
}


static void meiga_server_set_gui_pid (MeigaServer* self, guint value) {
	g_return_if_fail (self != NULL);
	self->priv->_gui_pid = value;
	g_object_notify ((GObject *) self, "gui-pid");
}


const char* meiga_server_get_display (MeigaServer* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_display;
	return result;
}


static void meiga_server_set_display (MeigaServer* self, const char* value) {
	char* _tmp1_;
	const char* _tmp0_;
	g_return_if_fail (self != NULL);
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	self->priv->_display = (_tmp1_ = (_tmp0_ = value, (_tmp0_ == NULL) ? NULL : g_strdup (_tmp0_)), self->priv->_display = (g_free (self->priv->_display), NULL), _tmp1_);
	g_object_notify ((GObject *) self, "display");
}


static void meiga_server_class_init (MeigaServerClass * klass) {
	meiga_server_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (MeigaServerPrivate));
	G_OBJECT_CLASS (klass)->get_property = meiga_server_get_property;
	G_OBJECT_CLASS (klass)->set_property = meiga_server_set_property;
	G_OBJECT_CLASS (klass)->finalize = meiga_server_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), MEIGA_SERVER_LOGGER, g_param_spec_object ("logger", "logger", "logger", TYPE_LOG, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), MEIGA_SERVER_GUI_PID, g_param_spec_uint ("gui-pid", "gui-pid", "gui-pid", 0, G_MAXUINT, (guint) 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), MEIGA_SERVER_DISPLAY, g_param_spec_string ("display", "display", "display", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("model_changed", TYPE_MEIGA_SERVER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void meiga_server_instance_init (MeigaServer * self) {
	self->priv = MEIGA_SERVER_GET_PRIVATE (self);
	self->priv->_logger = NULL;
	self->priv->_gui_pid = (guint) 0;
	self->priv->_display = NULL;
}


static void meiga_server_finalize (GObject* obj) {
	MeigaServer * self;
	self = MEIGA_SERVER (obj);
	(self->priv->server == NULL) ? NULL : (self->priv->server = (g_object_unref (self->priv->server), NULL));
	(self->priv->path_mapping == NULL) ? NULL : (self->priv->path_mapping = (g_hash_table_unref (self->priv->path_mapping), NULL));
	(self->priv->mimetypes == NULL) ? NULL : (self->priv->mimetypes = (g_hash_table_unref (self->priv->mimetypes), NULL));
	(self->priv->exposed == NULL) ? NULL : (self->priv->exposed = (g_object_unref (self->priv->exposed), NULL));
	(self->priv->net == NULL) ? NULL : (self->priv->net = (g_object_unref (self->priv->net), NULL));
	(self->priv->_logger == NULL) ? NULL : (self->priv->_logger = (g_object_unref (self->priv->_logger), NULL));
	self->priv->_display = (g_free (self->priv->_display), NULL);
	G_OBJECT_CLASS (meiga_server_parent_class)->finalize (obj);
}


GType meiga_server_get_type (void) {
	static GType meiga_server_type_id = 0;
	if (meiga_server_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (MeigaServerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) meiga_server_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (MeigaServer), 0, (GInstanceInitFunc) meiga_server_instance_init, NULL };
		meiga_server_type_id = g_type_register_static (G_TYPE_OBJECT, "MeigaServer", &g_define_type_info, 0);
	}
	return meiga_server_type_id;
}


static void meiga_server_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	MeigaServer * self;
	gpointer boxed;
	self = MEIGA_SERVER (object);
	switch (property_id) {
		case MEIGA_SERVER_LOGGER:
		g_value_set_object (value, meiga_server_get_logger (self));
		break;
		case MEIGA_SERVER_GUI_PID:
		g_value_set_uint (value, meiga_server_get_gui_pid (self));
		break;
		case MEIGA_SERVER_DISPLAY:
		g_value_set_string (value, meiga_server_get_display (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void meiga_server_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	MeigaServer * self;
	self = MEIGA_SERVER (object);
	switch (property_id) {
		case MEIGA_SERVER_LOGGER:
		meiga_server_set_logger (self, g_value_get_object (value));
		break;
		case MEIGA_SERVER_GUI_PID:
		meiga_server_set_gui_pid (self, g_value_get_uint (value));
		break;
		case MEIGA_SERVER_DISPLAY:
		meiga_server_set_display (self, g_value_get_string (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}


static void _vala_dbus_register_object (DBusConnection* connection, const char* path, void* object) {
	const _DBusObjectVTable * vtable;
	vtable = g_type_get_qdata (G_TYPE_FROM_INSTANCE (object), g_quark_from_static_string ("DBusObjectVTable"));
	if (vtable) {
		vtable->register_object (connection, path, object);
	} else {
		g_warning ("Object does not implement any D-Bus interface");
	}
}


static void _vala_dbus_unregister_object (gpointer connection, GObject* object) {
	char* path;
	path = g_object_steal_data ((GObject*) object, "dbus_object_path");
	dbus_connection_unregister_object_path (connection, path);
	g_free (path);
}




